/*#######################################################################################

AVR Embedded Webservice Project

File        : arp.c

Description : Implementation of the ARP protocol.

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#include "main.h"
#include "arp.h"

extern const unsigned char mymac[];
extern struct IPCONFIG * ipconfig;


extern unsigned long time;

unsigned char arp_cache_oldest = 0;

unsigned char ARP_Block = 0;

/*	Basic Receive-Function: Read and check ARP-Part of Packet and finish the rest to
	enable receiving of new packets.
*/
//############################################################################
unsigned char ARP_Receive(char *ARP_Buffer, int packetlen){
//############################################################################

	if (ARP_Block == 1){
		#if DEBUG_ARP
			printf("ARP blocked !\n");
		#endif
		return -1;
	}

	// Block ARP-Buffer
	ARP_Block = 1;

	#if DEBUG_CALLS
		print("-- ARP_Receive\n");
	#endif

	struct ARP_Packet *arp;
	arp = (struct ARP_Packet *) ARP_Buffer;
//	printf("%i.%i.%i.%i\n",dstip[0]dstip[1]dstip[2]dstip[3]);
	
	// If there are enough bytes left for a correct ARP-Packet read it into the Buffer
	if (packetlen >= ARP_SIZE){
		Read_Ethernet_Byte((unsigned char *)arp, ARP_SIZE);
		// Discard remaining bytes and re-open NIC for new packets
		Read_Ethernet_Terminate (packetlen-ARP_SIZE);
		// Check the Packet to see what it means.
		ARP_Check_Packet((unsigned char *)arp);
	}
	
	// Unblock ARP-Buffer
	ARP_Block = 0;
	return 0;
}

/*	Function to send the ARP-Packets.
	Request:
*/
//############################################################################
void ARP_Request(unsigned char * IP){
//############################################################################

	if (ARP_Block == 1){
		#if DEBUG_ARP
			printf("ARP blocked !\n");
		#endif
	}

	else {
		// Block ARP-Buffer
		ARP_Block = 1;

		#if DEBUG_ARP_SEND
			printf("-- ARP_Request for %i.%i.%i.%i\n",
				IP[0],
				IP[1],
				IP[2],
				IP[3]);
		#endif
		
		struct ARP_Packet *arp;
		arp = (struct ARP_Packet *)&reservedMemory[MEM_ARP_OFFSET];
		
		char type[2] = {0x08,ARP_TYPE};
		
		arp->HwType[0] = 0; arp->HwType[1] = 1;
		arp->ProtoType[0] = 8; arp->ProtoType[1] = 0;
		arp->HwSize = 6; arp->ProtoSize = 4;
		arp->Opcode[1] = 1;
		arp->DestMAC[0] = 255; arp->DestMAC[1] = 255;
		arp->DestMAC[2] = 255; arp->DestMAC[3] = 255;
		arp->DestMAC[4] = 255; arp->DestMAC[5] = 255;
		
		arp->SrcMAC[0] = mymac[0]; arp->SrcMAC[1] = mymac[1];
		arp->SrcMAC[2] = mymac[2]; arp->SrcMAC[3] = mymac[3];
		arp->SrcMAC[4] = mymac[4]; arp->SrcMAC[5] = mymac[5];
		
		arp->DestIP[0] = IP[0]; 
		arp->DestIP[1] = IP[1];
		arp->DestIP[2] = IP[2]; 
		arp->DestIP[3] = IP[3];

		arp->SrcIP[0] = ipconfig->ipAddress[0]; arp->SrcIP[1] = ipconfig->ipAddress[1];
		arp->SrcIP[2] = ipconfig->ipAddress[2]; arp->SrcIP[3] = ipconfig->ipAddress[3];
		
		#if DEBUG_ARP_SEND
			debug_print("Sending ", (unsigned char *)arp, ARP_SIZE);
		#endif
			
		Ethernet_Send (arp->DestMAC, type,
				   (unsigned char *)arp, ARP_SIZE);

		// Block ARP-Buffer
		ARP_Block = 0;
	}
}

/*	Function to parse the ARP-Packet.
*/
//############################################################################
void ARP_Check_Packet(unsigned char *ARP_Buffer){
//############################################################################
	
	struct ARP_Packet *arp;
	arp = (struct ARP_Packet *) ARP_Buffer;
	
	char type[2] = {0x08,ARP_TYPE};
	
	#if DEBUG_ARP_GET
		debug_print("Got ARP-Packet ", ARP_Buffer, ARP_SIZE);
	#endif
	
	// These are the values that need to match if we have a request/reply. All others
	// can be savely discarded
	if ((arp->HwType[0] == 0) && (arp->HwType[1] == 1) &&
		(arp->ProtoType[0] == 8) && (arp->ProtoType[1] == 0) &&
		(arp->HwSize == 6) && (arp->ProtoSize == 4)){
		
			if (arp->Opcode[0] == 0)
			{
				// Hey we have a request!
				if (arp->Opcode[1] == 1){
					// if it's for us send a reply (to be implemented) and add sender
					// to the cache
//					if (Byte_Compare(arp->DestIP, (unsigned char *)myip, IP_SIZE) == 0){
					if (IP_COMPARE(arp->DestIP,ipconfig->ipAddress)){
						#if DEBUG_ARP_GET
							debug_print("Request from ", arp->SrcMAC, MAC_SIZE);
							debug_print("Who is ", arp->DestIP, IP_SIZE);
						#endif
						ARP_Cache_Add((unsigned char *)arp);
						// Send Reply
						arp->Opcode[1] = 2;
						for (unsigned char ctr = 0; ctr < MAC_SIZE; ctr++){
							arp->DestMAC[ctr] = arp->SrcMAC[ctr];
							arp->SrcMAC[ctr] = mymac[ctr];
						}
						for (unsigned char ctr = 0; ctr < IP_SIZE; ctr++){
							arp->DestIP[ctr] = arp->SrcIP[ctr];
							arp->SrcIP[ctr] = ipconfig->ipAddress[ctr];
						}
						
						#if DEBUG_ARP_SEND
							debug_print("Sending ", (unsigned char *)arp, ARP_SIZE);
						#endif
						
						Ethernet_Send (arp->DestMAC, type,
									   (unsigned char *)arp, ARP_SIZE);
					}
					// if not forget about it
					#if DEBUG_ARP_GET
						else debug_print("Not for me     ", arp->DestIP, IP_SIZE);
					#endif
				}
				// It's a reply, good we have (probably) been waiting for this. 
				// Add Sender to the
				// cache.
				else if (arp->Opcode[1] == 2){
					#if DEBUG_ARP_GET
						debug_print("Reply from     ", arp->SrcMAC, MAC_SIZE);
					#endif
					ARP_Cache_Add((unsigned char *)arp);
				}
				#if DEBUG_ARP_GET
					else printf("Unknown Opcode: %02x", arp->Opcode[1]);
				#endif
			}
	}
}

/*	Function to check if a given IP has an entry in the ARP-Cache. Returns:
	- NULL if no entry was found (ARP-Request has to be sent)
	- a Pointer to the corresponding MAC-Adress
*/
//############################################################################
unsigned char *ARP_Cache_Check(unsigned char* IP){
//############################################################################
	#if DEBUG_ARP_CACHE
		// this debug message is not part of DEBUG_CALLS because its called
		// too frequently
		printf("-- ARP_Cache_Check %i.%i.%i.%i\n",
			IP[0],
			IP[1],
			IP[2],
			IP[3]);
	#endif
	
	unsigned char ctr;

	//	Check each entry in the ARP-Cache
	for (ctr = 0; ctr < ARP_CACHE_SIZE; ctr++){
		// If we have a match that isnt outdated return *MAC-Pointer
		if ((Byte_Compare(arp_cache[ctr].IP, IP, IP_SIZE) == 0) &&
			((arp_cache[ctr].timestamp + ARP_CACHE_TIMEOUT) > time)){
			#if DEBUG_ARP_CACHE
				print("Cache Hit\n");
			#endif
			return (unsigned char *)arp_cache[ctr].MAC;
		}
	}
	#if DEBUG_ARP_CACHE
		print("Cache Miss\n");
	#endif
	return NULL;
}

/*	Function to add if a given MAC/IP-Combination to the ARP-Cache.
*/
//############################################################################
void ARP_Cache_Add(unsigned char *ARP_Buffer){
//############################################################################
	
	unsigned char ctr, ctr2;
	struct ARP_Packet *arp;
	arp = (struct ARP_Packet *) ARP_Buffer;
		
	//	Check each entry in the ARP-Cache
	for (ctr = 0; ctr < ARP_CACHE_SIZE; ctr++){
		// if entry isn't empty but has the same MAC that is to be added, replace the
		// IP with the new one (no matter if they are the same).
		if (Byte_Compare(arp_cache[ctr].MAC, arp->SrcMAC, MAC_SIZE) == 0){
			for (ctr2 = 0; ctr2 < IP_SIZE; ctr2++){
				arp_cache[ctr].IP[ctr2] = arp->SrcIP[ctr2];
			}
			arp_cache[ctr].timestamp = time;
			#if DEBUG_ARP_CACHE
				printf("Add to Slot %d ", ctr);
				debug_print("", arp->SrcMAC, MAC_SIZE+IP_SIZE+1);
			#endif
			break;
		}
		// if no matching entry was found check for oldest entry and replace it
		else {
			unsigned char oldest_entry = 0;
			
			for (ctr2 = 0; ctr2 < ARP_CACHE_SIZE; ctr2++){
				if (arp_cache[ctr2].timestamp < arp_cache[oldest_entry].timestamp){
					oldest_entry = ctr2;
				}
			}
			for (ctr2 = 0; ctr2 < MAC_SIZE; ctr2++){
				arp_cache[oldest_entry].MAC[ctr2] = arp->SrcMAC[ctr2];
			}
			for (ctr2 = 0; ctr2 < IP_SIZE; ctr2++){
				arp_cache[oldest_entry].IP[ctr2] = arp->SrcIP[ctr2];
			}
			arp_cache[oldest_entry].timestamp = time;
		}
		#if DEBUG_ARP_CACHE
			printf("Add to Slot %d ", ctr);
			debug_print("", arp->SrcMAC, MAC_SIZE+IP_SIZE+1);
		#endif
		break;
	}
}

/*	Simple Function to compare two given Byte-streams over a given length. Returns:
	- 0 is they are the same
	- -1 if not
*/
//############################################################################
unsigned char Byte_Compare(unsigned char *buf1, unsigned char *buf2, unsigned char len){
//############################################################################

	unsigned char result = 0;
	unsigned char ctr;
	
	for (ctr = 0; ctr < len; ctr++){
		if (buf1[ctr] != buf2[ctr]){
			result = -1;
			break;
		}
	}
	
	return result;
}
