/*#######################################################################################

AVR Embedded Webservice Project

File        : arp.h

Description : Implementation of the ARP protocol (header).

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#ifndef _ARP_H
#define _ARP_H

#include "ethernet.h"

#define ARP_TYPE 6
#define ARP_SIZE 28
#define ARP_CACHE_SIZE 3
#define ARP_CACHE_TIMEOUT 60

#define MAC_SIZE 6
#define IP_SIZE 4

#define ARP_RESOLVE_IP(x) ARP_Cache_Check((unsigned char*)x)

extern unsigned char ARP_Receive(char *ARP_Buffer, int packetlen);
extern void ARP_Request(unsigned char *IP);
extern void ARP_Check_Packet(unsigned char *ARP_Buffer);
extern unsigned char *ARP_Cache_Check(unsigned char* IP);
extern void ARP_Cache_Add(unsigned char *ARP_Buffer);
extern unsigned char Byte_Compare(unsigned char *buf1, unsigned char *buf2, unsigned char len);



struct ARP_Packet {
	unsigned char HwType[2];		// Ethernet:0x0001
	unsigned char ProtoType[2];		// IP:0x0800
	unsigned char HwSize;			// Laenge der MAC Adresse: 6
	unsigned char ProtoSize;		// Laenge der IP Adresse: 4
	unsigned char Opcode[2];		// Request:0x0001, Reply:0x0002
	unsigned char SrcMAC[MAC_SIZE];
	unsigned char SrcIP[IP_SIZE];
	unsigned char DestMAC[MAC_SIZE];
	unsigned char DestIP[IP_SIZE];
};

struct ARP_Cache {
	unsigned char MAC[MAC_SIZE];
	unsigned char IP[IP_SIZE];
	unsigned long timestamp;
};

#endif //_ARP_H
