/*#######################################################################################

AVR Embedded Webservice Project

File        : clock.c

Description : Global clock counter.

Copyright notice:

Copyright (C) 2004 - Ulrich Radig (originally written for AVR Small Webserver).
few changes made by
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#include "main.h"
#include "clock.h"

// Diese Timer werden bald rausgeschmissen - wenn dhcp auch den globalen Timer nutzt
unsigned char ss	= 0; //Variable fuer Sekunden
unsigned char mm	= 21;//Variable fuer Minuten
unsigned char hh	= 22;//Variable fuer Stunden

// Global time-counter. Counts seconds since last Reset
unsigned long time = 0;
unsigned char time_short = 0;


#if USE_CLOCK

//############################################################################
//Diese Routine addiert 1Sekunde wenn Timer Interrupt ausgeloest wird 
SIGNAL (SIG_OVERFLOW1)
//############################################################################
{
	
	DHCP_Timer();
	
	#if DEBUG_CLOCK
		printf("Timer:%02i:%02i:%02i\n",hh,mm,ss);
		// Timer LED
		if (ledstat(2)) ledoff(2); else ledon(2);
	#endif
	
	#if DEBUG_ETHERNET
		// Activity LED
		ledoff(0);
	#endif
	
	TCNT1 = 65535 - (SYSCLK / 1024);
	time++;
	time_short++; //Addiere +1
	ss++;
	if (ss == 60){
		ss = 0;
		mm++;//Addiere +1
		if (mm == 60){
			mm = 0;
			hh++;//Addiere +1
			if (hh == 24){
				hh = 0;
			}
		}	
	}		
}


//############################################################################
//Diese Routine startet und inizialisiert den Timer
void Start_Clock (void)
//############################################################################
{
	//Interrupt for the Clock enable
	timer_enable_int(_BV(TOIE1));
	//Setzen des Prescaler auf 1024 
	TCCR1B |= (1<<CS10 | 0<<CS11 | 1<<CS12); 
	return;
}

#endif //#if USE_CLOCK

