/*#######################################################################################

AVR Embedded Webservice Project

File        : dhcp.c

Description : Implementation of DHCP client-services.

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#include "main.h"
#include "dhcp.h"
#include "udp.h"

struct UDP_SOCKET *dhcp_broadcast, *dhcp_listen;

unsigned char dhcp_retries = 0;

//############################################################################
void DHCP_Timer(void){
//############################################################################
	int i=0;
	// start DHCP init on system startup
	if ((dhcp_retries<DHCP_MAX_RETRIES) 
	   && (dhcp_config.dhcp_state!=DHCP_STAT_BOUND)) {
		#if DEBUG_DHCP
			printf("DHCP: try to resolv IP [%i]:\n",dhcp_retries);
		#endif
		dhcp_retries++;
		DHCP_Init();
	}
	
	// reset dhcp_retries and signal IP
	if ((dhcp_config.dhcp_state==DHCP_STAT_BOUND) && (dhcp_retries>0)) {
		#if DHCP_SIGNAL_IP
			if (dhcp_retries>0 && dhcp_retries<=DHCP_MAX_RETRIES){
				dhcp_retries = DHCP_MAX_RETRIES;
			}
			
			// SIGNAL LED2
			if (dhcp_retries<DHCP_MAX_RETRIES+(ipconfig->ipAddress[3]/100)%10){
				ledon(2);
			}

			// SIGNAL LED1
			if (dhcp_retries<DHCP_MAX_RETRIES+(ipconfig->ipAddress[3]/10)%10){
				ledon(1);
			}

			// SIGNAL LED0
			if (dhcp_retries<DHCP_MAX_RETRIES+(ipconfig->ipAddress[3])%10){
				ledon(0);
			}

			for (i=0;i<30000;i++);
			ledoff(0);ledoff(1);ledoff(2);
			
			dhcp_retries++;	
			
			// quit after maximum of 10 flashes
			if (dhcp_retries>DHCP_MAX_RETRIES+10) dhcp_retries = 0;
			
		#else
			dhcp_retries = 0;
		#endif
	}

}

//############################################################################
void DHCP_Receive(unsigned char* packet, int packetlen){
//############################################################################
	int i, c;
	#if DEBUG_CALLS
		print("---- DHCP_RECEIVE\n");
	#endif
	// states: 
	// 0: we have sent nothing
	// 1: we have sent a DHCP_DISCOVER message
	// 2: we have received a DHCP_OFFER
	// 3: we have sent a DHCP_REQUEST
	// 4: we have received a DHCP_ACK	

	struct DHCP_Packet *dhcp;
	dhcp = (struct DHCP_Packet*) packet;

	// check Packet
	#if DEBUG_DHCP
		printf("DHCP State: %i\n",dhcp_config.dhcp_state);
	#endif
	//*********************************************************************
	if(dhcp_config.dhcp_state==DHCP_STAT_DISCOVER){
	//*********************************************************************
		// check for DHCP_OFFER
		if(dhcp->op!=DHCP_BOOTREPLY) {
			#if DEBUG_DHCP
				printf("not a reply\n");
			#endif
			DHCP_FAIL();
		}
		// scan Options for Messagetype(53)==DHCP Offer(02)
		i=0;
		while ((i<(sizeof(dhcp->options)-2)) && (i>=0)) {
			// check option at point i
			if ((dhcp->options[i]==DHCP_OPTION_MSGTYPE)
			  &&(dhcp->options[i+1]==1)
			  &&(dhcp->options[i+2]==DHCP_OPTION_MSGTYPE_OFFER)){
			  	// set i=-1 as hint that search was successful
				i=sizeof(dhcp->options);
			} else{
				// advance to next option
				i+=dhcp->options[i+1]+2;
			}
		}
		if (i<sizeof(dhcp->options)) {
			#if DEBUG_DHCP
				printf("Messagetype not DHCP OFFER\n");
			#endif
			DHCP_FAIL();
		}
		
		// is this message a reply to our request (xid)?
		if ((CHAR2LONG(dhcp->xid))!=CHAR2LONG(dhcp_config.xid)) {
			#if DEBUG_DHCP
				printf("xid doesn't match\n");
			#endif
			DHCP_FAIL();
		}
		
		// .. guess the offer is for us, lets build a request to accept
		dhcp->op=DHCP_BOOTREQUEST; 
		dhcp->ciaddr[0]=dhcp->yiaddr[0];
		dhcp->ciaddr[1]=dhcp->yiaddr[1];
		dhcp->ciaddr[2]=dhcp->yiaddr[2];
		dhcp->ciaddr[3]=dhcp->yiaddr[3];
		dhcp->yiaddr[0]=0;dhcp->yiaddr[1]=0;dhcp->yiaddr[2]=0;dhcp->yiaddr[3]=0;
		dhcp->siaddr[0]=0;dhcp->siaddr[1]=0;dhcp->siaddr[2]=0;dhcp->siaddr[3]=0;
		dhcp->giaddr[0]=0;dhcp->giaddr[1]=0;dhcp->giaddr[2]=0;dhcp->giaddr[3]=0;
		dhcp->chaddr[0]=mymac[0];dhcp->chaddr[1]=mymac[1];dhcp->chaddr[2]=mymac[2];
		dhcp->chaddr[3]=mymac[3];dhcp->chaddr[4]=mymac[4];dhcp->chaddr[5]=mymac[5];
		dhcp->secs[0]=0;dhcp->secs[0]=0;
		dhcp->flags[0]=0;
		dhcp->flags[1]=0;


		// reset Options
		// look for server identifier
		c=0;
		for (i=0;i<sizeof(dhcp->options) && (c==0);i++){
			// set message type
			if (dhcp->options[i]==DHCP_OPTION_SERVERID){
				c=i;
			}
			i+=dhcp->options[i+1]+1;
		}
		// set server identifier assuming that Messagetype will be set in front
		// of it.

		#if DEBUG_DHCP
			printf("ServerId at: %i, copy:",c);
		#endif

		if (c==12){ /*its already in place, do nothing*/ }
		else if (c<12) {
			// copy it, starting at the end to not overwrite the beginning
			for (i=1+dhcp->options[c+1];i>=0;i--){
				#if DEBUG_DHCP
					printf("%02x",dhcp->options[c+i]);
				#endif
				dhcp->options[12+i]=dhcp->options[c+i];
			}
		} else if (c>12){
			// copy it, starting at the beginning to not overwrite the end
			for (i=0;i<=1+dhcp->options[c+1];i++){
				dhcp->options[12+i]=dhcp->options[c+i];
			}
		}
		// set Messagetype
		dhcp->options[0]=DHCP_OPTION_MSGTYPE;
		dhcp->options[1]=1;
		dhcp->options[2]=DHCP_OPTION_MSGTYPE_REQUEST;
		dhcp->options[3]=DHCP_OPTION_CLIENT_IDENTIFIER;
		dhcp->options[4]=7;
		dhcp->options[5]=DHCP_10mbitETHERNET;
		dhcp->options[6]=mymac[0];
		dhcp->options[7]=mymac[1];
		dhcp->options[8]=mymac[2];
		dhcp->options[9]=mymac[3];
		dhcp->options[10]=mymac[4];
		dhcp->options[11]=mymac[5];
		c = 14+dhcp->options[13];

		dhcp->yiaddr[0]=0;dhcp->yiaddr[1]=0;dhcp->yiaddr[2]=0;dhcp->yiaddr[3]=0;
		dhcp->options[c++]=DHCP_OPTION_HOST_NAME;
		dhcp->options[c++]=3;
		dhcp->options[c++]='A';
		dhcp->options[c++]='V';
		dhcp->options[c++]='R';
		dhcp->options[c++]=DHCP_OPTION_VENDOR_CLASSID;
		dhcp->options[c++]=3;
		dhcp->options[c++]='J';
		dhcp->options[c++]='O';
		dhcp->options[c++]='N';
		dhcp->options[c++]=DHCP_OPTION_PARAMREQLIST;
		dhcp->options[c++]=2;	// !!! increase size when adding params
		dhcp->options[c++]=DHCP_OPTION_SUBNETMASK;
//		dhcp->options[c++]=DHCP_OPTION_DOMAINNAME;
		dhcp->options[c++]=DHCP_OPTION_ROUTER;
//		dhcp->options[c++]=DHCP_OPTION_DNS;
//		dhcp->options[c++]=DHCP_OPTION_ROUTERDSCVR;		
		dhcp->options[c++]=DHCP_OPTION_END;
		
		// Fill Padding
		for(i=c;i<sizeof(dhcp->options);dhcp->options[i++]=DHCP_OPTION_PADDING);
		
		
		
		// Send
//		printf("DHCP_XID: %c%c%c%c",dhcp_buffer[4],dhcp_buffer[5],dhcp_buffer[6],dhcp_buffer[7]);
		dhcp_config.dhcp_state=DHCP_STAT_REQUEST;

		UDP_Send(dhcp_broadcast,dhcp_listen,(unsigned char*) dhcp,300);		

	}

	//*********************************************************************
	if(dhcp_config.dhcp_state==DHCP_STAT_REQUEST){
	//*********************************************************************
		// check for DHCP_ACK and edit ipconfig
		#if DEBUG_DHCP
			print("Receiving ACK? ");
		#endif
		if(dhcp->op!=DHCP_BOOTREPLY) {
			#if DEBUG_DHCP
				printf("not a reply!\n");
			#endif
			DHCP_FAIL();
		}
		// scan Options for Messagetype(53)==DHCP ACK(5)
		i=0;
		while ((i<(sizeof(dhcp->options)-2)) && (i>=0)) {
			// check option at point i
			if ((dhcp->options[i]==DHCP_OPTION_MSGTYPE)
			  &&(dhcp->options[i+1]==1)
			  &&(dhcp->options[i+2]==DHCP_OPTION_MSGTYPE_ACK)){
			  	// set i=-1 as hint that search was successful
				i=sizeof(dhcp->options);
			} else{
				// advance to next option
				i+=dhcp->options[i+1]+2;
			}
		}
		if (i<sizeof(dhcp->options)) {
			#if DEBUG_DHCP
				printf("Messagetype not DHCP ACK!\n");
			#endif
			DHCP_FAIL();
		}
		
		// is this message a reply to our request (xid)?
		if ((CHAR2LONG(dhcp->xid))!=CHAR2LONG(dhcp_config.xid)) {
			#if DEBUG_DHCP
				printf("xid doesn't match!\n");
			#endif
			DHCP_FAIL();
		}

		#if DEBUG_DHCP
			print("YAY!! saving IP Configuration\n");
		#endif
		// save configuration
		// IP Address
		ipconfig->ipAddress[0]=dhcp->ciaddr[0];
		ipconfig->ipAddress[1]=dhcp->ciaddr[1];
		ipconfig->ipAddress[2]=dhcp->ciaddr[2];
		ipconfig->ipAddress[3]=dhcp->ciaddr[3];
		
		// Subnet Mask
		c=0;
		for (i=0;i<sizeof(dhcp->options) && (c==0);i++){
			// set message type
			if (dhcp->options[i]==DHCP_OPTION_SUBNETMASK){
				ipconfig->netMask[0]=dhcp->options[i+2];
				ipconfig->netMask[1]=dhcp->options[i+3];
				ipconfig->netMask[2]=dhcp->options[i+4];
				ipconfig->netMask[3]=dhcp->options[i+5];
				c=1;
			}
			i+=dhcp->options[i+1]+1;
		}
		
		// Gateway (Router)
		c=0;
		for (i=0;i<sizeof(dhcp->options) && (c==0);i++){
			// set message type
			if (dhcp->options[i]==DHCP_OPTION_ROUTER){
				ipconfig->gateway[0]=dhcp->options[i+2];
				ipconfig->gateway[1]=dhcp->options[i+3];
				ipconfig->gateway[2]=dhcp->options[i+4];
				ipconfig->gateway[3]=dhcp->options[i+5];
				c=1;
			}
			i+=dhcp->options[i+1]+1;
		}


		#if DEBUG_DHCP
			printf(" ipAddress : %i.%i.%i.%i\n",
				ipconfig->ipAddress[0],ipconfig->ipAddress[1],
				ipconfig->ipAddress[2],ipconfig->ipAddress[3]
			);
			printf(" SubnetMask: %i.%i.%i.%i\n",
				ipconfig->netMask[0],ipconfig->netMask[1],
				ipconfig->netMask[2],ipconfig->netMask[3]
			);
			printf(" Gateway   : %i.%i.%i.%i\n",
				ipconfig->gateway[0],ipconfig->gateway[1],
				ipconfig->gateway[2],ipconfig->gateway[3]
			);
		#endif


		dhcp_config.dhcp_state = DHCP_STAT_BOUND;
		
		// Set TCP Listening Socket
		tcp_sockets[tcp_connections[0].localSocket].ipAddr[0] = ipconfig->ipAddress[0];
		tcp_sockets[tcp_connections[0].localSocket].ipAddr[1] = ipconfig->ipAddress[1];
		tcp_sockets[tcp_connections[0].localSocket].ipAddr[2] = ipconfig->ipAddress[2];
		tcp_sockets[tcp_connections[0].localSocket].ipAddr[3] = ipconfig->ipAddress[3];
		
		ledoff(1);
			
	}
}

//############################################################################
void DHCP_Init(void){
//############################################################################
	#if DEBUG_CALLS
		print("---- DHCP_INIT\n");
	#endif
	struct DHCP_Packet* dhcp;

	if (dhcp_retries==0) dhcp_retries=1;
	dhcp_config.dhcp_state = DHCP_STAT_DISCOVER;
	
	int i=0;
	// generate Transaction ID
	dhcp_config.xid[0]=hh-mm+ss;
	dhcp_config.xid[1]=ss+mm-hh;
	dhcp_config.xid[2]=ss+5;
	dhcp_config.xid[3]=mm-ss;
	
	// DHCP DISCOVER PACKET erzeugen
	dhcp = (struct DHCP_Packet *) &reservedMemory[MEM_DHCP_OFFSET];
	dhcp->op=DHCP_BOOTREQUEST; 
	dhcp->htype=DHCP_10mbitETHERNET;
	dhcp->hlen=DHCP_10mbitETHERNETLEN;
	dhcp->hops=0;
	dhcp->xid[0]=dhcp_config.xid[0];dhcp->xid[1]=dhcp_config.xid[1];
	dhcp->xid[2]=dhcp_config.xid[2];dhcp->xid[3]=dhcp_config.xid[3];
	dhcp->secs[0]=0;dhcp->secs[0]=0;
	dhcp->flags[0]=0;
	dhcp->flags[1]=0;
	dhcp->ciaddr[0]=0;dhcp->ciaddr[1]=0;dhcp->ciaddr[2]=0;dhcp->ciaddr[3]=0;
	dhcp->yiaddr[0]=0;dhcp->yiaddr[1]=0;dhcp->yiaddr[2]=0;dhcp->yiaddr[3]=0;
	dhcp->siaddr[0]=0;dhcp->siaddr[1]=0;dhcp->siaddr[2]=0;dhcp->siaddr[3]=0;
	dhcp->giaddr[0]=0;dhcp->giaddr[1]=0;dhcp->giaddr[2]=0;dhcp->giaddr[3]=0;
	dhcp->chaddr[0]=mymac[0];dhcp->chaddr[1]=mymac[1];dhcp->chaddr[2]=mymac[2];
	dhcp->chaddr[3]=mymac[3];dhcp->chaddr[4]=mymac[4];dhcp->chaddr[5]=mymac[5];
	for(i=6;i<15;i++) dhcp->chaddr[i]=0;
//		dhcp->sname[0]='A';dhcp->sname[1]='V';dhcp->sname[2]='R';dhcp->sname[3]='\0';
	for(i=0;i<63;i++) dhcp->sname[i]=0;
	for(i=0;i<127;i++) dhcp->file[i]=0;
	// Magic Cookie
	dhcp->magic_cookie[0]=99;
	dhcp->magic_cookie[1]=130;
	dhcp->magic_cookie[2]=83;
	dhcp->magic_cookie[3]=99;
	int a=0;
	// Options
	dhcp->options[a++]=DHCP_OPTION_MSGTYPE;
	dhcp->options[a++]=1;
	dhcp->options[a++]=DHCP_OPTION_MSGTYPE_DISCOVER;
	dhcp->options[a++]=DHCP_OPTION_CLIENT_IDENTIFIER;
	dhcp->options[a++]=7;
	dhcp->options[a++]=DHCP_10mbitETHERNET;
	dhcp->options[a++]=mymac[0];
	dhcp->options[a++]=mymac[1];
	dhcp->options[a++]=mymac[2];
	dhcp->options[a++]=mymac[3];
	dhcp->options[a++]=mymac[4];
	dhcp->options[a++]=mymac[5];
/*	dhcp->options[a++]=DHCP_OPTION_AUTO_CONFIGURE;
	dhcp->options[a++]=1;
	dhcp->options[a++]=1;*/
	dhcp->options[a++]=DHCP_OPTION_HOST_NAME;
	dhcp->options[a++]=3;
	dhcp->options[a++]='A';
	dhcp->options[a++]='V';
	dhcp->options[a++]='R';
	dhcp->options[a++]=DHCP_OPTION_VENDOR_CLASSID;
	dhcp->options[a++]=3;
	dhcp->options[a++]='J';
	dhcp->options[a++]='O';
	dhcp->options[a++]='N';
	dhcp->options[a++]=DHCP_OPTION_PARAMREQLIST;
	dhcp->options[a++]=2;	// !!! increase size when adding params
	dhcp->options[a++]=DHCP_OPTION_SUBNETMASK;
//	dhcp->options[a++]=DHCP_OPTION_DOMAINNAME;
	dhcp->options[a++]=DHCP_OPTION_ROUTER;
//	dhcp->options[a++]=DHCP_OPTION_DNS;
//	dhcp->options[a++]=DHCP_OPTION_ROUTERDSCVR;
	dhcp->options[a++]=DHCP_OPTION_END;
	// add padding
	for(i=a;i<sizeof(dhcp->options);dhcp->options[i++]=DHCP_OPTION_PADDING);		
	// create socket for sending and receiving
	
	dhcp_broadcast=&(udp_sockets[0]);
	dhcp_listen=&(udp_sockets[1]);

	dhcp_broadcast->ip[0]=255;dhcp_broadcast->ip[1]=255;
	dhcp_broadcast->ip[2]=255;dhcp_broadcast->ip[3]=255;
	dhcp_broadcast->port[0]=0;dhcp_broadcast->port[1]=UDP_PORT_DHCPSERVER;

	dhcp_listen->ip[0]=0;dhcp_listen->ip[1]=0;
	dhcp_listen->ip[2]=0;dhcp_listen->ip[3]=0;
	dhcp_listen->port[0]=0;dhcp_listen->port[1]=UDP_PORT_DHCPCLIENT;

//		printf("DHCP_XID: %c%c%c%c",dhcp_buffer[4],dhcp_buffer[5],dhcp_buffer[6],dhcp_buffer[7]);
	UDP_Send(dhcp_broadcast,dhcp_listen,(unsigned char*) dhcp,300);		
}
