/*#######################################################################################

AVR Embedded Webservice Project

File        : dhcp.h

Description : Implementation of DHCP client-services (header).

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#ifndef _DHCP_H_
#define _DHCP_H_

#define DHCP_SIGNAL_IP          1
#define DHCP_MAX_RETRIES        10

#define DHCP_BOOTREQUEST        1
#define DHCP_BOOTREPLY		    2
#define DHCP_10mbitETHERNET		1
#define DHCP_10mbitETHERNETLEN	6
#define DHCP_SIZE             300
#define DHCP_FAIL() return;

#define DHCP_STAT_DISCOVER  1
#define DHCP_STAT_REQUEST   2
#define DHCP_STAT_FAILED    3
#define DHCP_STAT_BOUND     4


#define DHCP_OPTION_PADDING              0
#define DHCP_OPTION_SUBNETMASK           1
#define DHCP_OPTION_ROUTER               3
#define DHCP_OPTION_DNS                  6
#define DHCP_OPTION_HOST_NAME            12
#define DHCP_OPTION_DOMAINNAME           15
#define DHCP_OPTION_ROUTERDSCVR          31
#define DHCP_OPTION_REQUEST_IP           50
#define DHCP_OPTION_LEASETIME            51
#define DHCP_OPTION_MSGTYPE              53
#define   DHCP_OPTION_MSGTYPE_DISCOVER     1
#define   DHCP_OPTION_MSGTYPE_OFFER        2
#define   DHCP_OPTION_MSGTYPE_REQUEST      3
#define   DHCP_OPTION_MSGTYPE_ACK          5
#define DHCP_OPTION_SERVERID             54
#define DHCP_OPTION_PARAMREQLIST         55
#define DHCP_OPTION_VENDOR_CLASSID       60
#define DHCP_OPTION_CLIENT_IDENTIFIER    61
#define DHCP_OPTION_AUTO_CONFIGURE       116
#define DHCP_OPTION_END                  255


struct DHCP_Packet {
	unsigned char op;
	unsigned char htype;
	unsigned char hlen;
	unsigned char hops;
	unsigned char xid[4];
	unsigned char secs[2];
	unsigned char flags[2];
	unsigned char ciaddr[4];
	unsigned char yiaddr[4];
	unsigned char siaddr[4];
	unsigned char giaddr[4];
	unsigned char chaddr[16];	
	unsigned char sname[64];
	unsigned char file[128];
	unsigned char magic_cookie[4];
	unsigned char options[60];
};

struct DHCP_Config {
	unsigned char dhcp_state;
	unsigned char xid[4];
}

extern dhcp_config;

/*
extern struct UDP_SOCKET* dhcp_broadcast;
extern struct UPD_SOCKET* dhcp_listen;
*/


extern void DHCP_Receive(unsigned char* packet, int packetlen);
extern void DHCP_Init(void);
extern void DHCP_Timer(void);
#endif //_DHCP_H_
