/*#######################################################################################

AVR Embedded Webservice Project

File        : ethernet.c

Description : Reading and sending between high-/low-level protocols.

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de
based on code from Ulrich Radig for the AVR Small Webserver 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#include "main.h"
#include "ethernet.h"

MAC_ADDRESS mac_broadcast_addr={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

//############################################################################
//Hier werden daten empfangen und geprueft was empfangen wurde 
int Ethernet_Receive (void)
//############################################################################
{
	#if DEBUG_CALLS
		print("-- Ethernet_Receive\n");
	#endif
	
	#if DEBUG_ETHERNET
		print("Ethernet_Receive 1 .. ");
	#endif
	//laenge des Ethernetpacketes
	unsigned int packetlen;
		
	#if DEBUG_ETHERNET
		print("2 .. ");
	#endif
	unsigned char waiter = 0;
	unsigned char *packet;
	packet = &reservedMemory[MEM_ETHER_OFFSET];
	#if DEBUG_ETHERNET
		print("3 .. ");
	#endif
	
	
	//Pointer auf Ethernet_Header
	struct Ethernet_Header *ethernet;
	ethernet = (struct Ethernet_Header *)&reservedMemory[MEM_ETHER_OFFSET];
	#if DEBUG_ETHERNET
		printf("4!\n");
	#endif
	
	//Lese daten aus NIC FIFO ins Array packet() sowie packetheader()
	packetlen = Read_Ethernet_Init();
	#if DEBUG_ETHERNET && DEBUG_VERBOSE
		printf("Packet Len: %i", packetlen);
	#endif		
	if (packetlen<=ETHER_HEADER_SIZE) return 0;
	Read_Ethernet_Byte (packet, ETHER_HEADER_SIZE);

	#if DEBUG_ETHERNET && DEBUG_VERBOSE
		debug_print("EthernetHeader:", (unsigned char *)ethernet, ETHER_HEADER_SIZE);
	#endif		
		
	if (ethernet->EnetPacketType[0] == 8){
		switch(ethernet->EnetPacketType[1]){
			case 0:{
				#if DEBUG_ETHERNET
					debug_print("Packet Type:    IP", ethernet->EnetPacketType, 2);
				#endif
				waiter = IP_Receive(&packet[MEM_IP_RECEIVE_OFFSET],
									packetlen-ETHER_HEADER_SIZE);

				break;
			}
			case 6:{
				#if DEBUG_ETHERNET
					debug_print("Packet Type:    ARP", ethernet->EnetPacketType, 2);
				#endif
				waiter = ARP_Receive(&packet[MEM_ARP_OFFSET],
									 packetlen-ETHER_HEADER_SIZE);
				break;
			}
			default:{
				#if DEBUG_ETHERNET
					debug_print("Packet Type:    sonstiges",ethernet->EnetPacketType, 2);
				#endif
				Read_Ethernet_Terminate (packetlen-ETHER_HEADER_SIZE);
			}
		}
	}
	else Read_Ethernet_Terminate (packetlen-ETHER_HEADER_SIZE);
	
	return (0);
}

//############################################################################
//Hier werden daten gesammelt und gesendet was empfangen wurde 
void Ethernet_Send (unsigned char *Packet_Dest, unsigned char Packet_Type[2], unsigned char *Packet_Data, unsigned int Packet_Length)
//############################################################################
{
	#if DEBUG_CALLS
		print("-- Ethernet_Send\n");
	#endif
	
	unsigned char ctr;
	
	struct Ethernet_Header *ethernet;
	ethernet = (struct Ethernet_Header *)&reservedMemory[MEM_ETHER_OFFSET];
	
	for (ctr = 0; ctr < MAC_SIZE; ctr++){
		ethernet->EnetPacketDest[ctr] = Packet_Dest[ctr];
		ethernet->EnetPacketSrc[ctr] = mymac[ctr];
	}
	ethernet->EnetPacketType[0] = Packet_Type[0];
	ethernet->EnetPacketType[1] = Packet_Type[1];
	
	Write_Ethernet_Init(ETHER_HEADER_SIZE+Packet_Length);

	Write_Ethernet_Buffer((unsigned char *)ethernet, ETHER_HEADER_SIZE);
	Write_Ethernet_Buffer(Packet_Data, Packet_Length);

	Write_Ethernet_Terminate(ETHER_HEADER_SIZE+Packet_Length);	
}

char InterruptCounter = 0;

//############################################################################
#if defined (__AVR_ATmega32__)	
SIGNAL (SIG_INTERRUPT0);
#endif	
#if defined (__AVR_ATmega103__)
SIGNAL (SIG_INTERRUPT4);
#endif	
#if defined (__AVR_ATmega128__)
SIGNAL (SIG_INTERRUPT4);
#endif

#if defined (__AVR_ATmega32__)	
SIGNAL (SIG_INTERRUPT0)
#endif	

#if defined (__AVR_ATmega103__)
SIGNAL (SIG_INTERRUPT4)
#endif

#if defined (__AVR_ATmega128__)
SIGNAL (SIG_INTERRUPT4)
#endif
{
	NIC_INT=1;
	char c=InterruptCounter++;
//	return 0;
	//Globale Interrupts Ausschalten
	cli ();
	
	
	#if DEBUG_ETHERNET
		// Activity LED
		ledon(0);
	#endif
	
	#if DEBUG_CALLS
		print("\n");
		printf("<Interrupt id=\"%i\">START %i!!\n",c,c);
	#endif

	char ByteH = 0;
	char ByteL = 1;		
	
	//if the receive buffer has been overrun
	if ( (ReadRTL(ISR)&(1<<OVW)) != 0)
	{
		print ("Overrun!\n");
	}
	

	//gehe zu Routine Ethernet_Receive wenn ein Packet empfangen wurde
	if ( (ReadRTL(ISR) & (1<<PRX)) != 0)
	{
//		Ethernet_Receive();
	
		while (ByteL != ByteH) //(!= bedeutet ungleich)
		{
			Ethernet_Receive();
			// disable interrupts so that the following operations are not
			// interupted until Data is read
			cli();
			//auslesen des Empfangsbuffer BNRY = CURR 
			ByteL = ReadRTL(BNRY); //auslesen NIC Register bnry
			WriteRTL ( CR ,(1<<STA|1<<RD2|1<<PS0));
			
			ByteH = ReadRTL(CURR); //auslesen NIC Register curr
			WriteRTL ( CR ,(1<<STA|1<<RD2));
		} 


	}
	#if DEBUG_ETHERNET
		else print("Kein Paket.\n");
	#endif




/*		
	//auslesen des Empfangsbuffer BNRY = CURR 
	ByteL = ReadRTL(BNRY); //auslesen NIC Register bnry
	WriteRTL ( CR ,(1<<STA|1<<RD2|1<<PS0));
	
	ByteH = ReadRTL(CURR); //auslesen NIC Register curr
	WriteRTL ( CR ,(1<<STA|1<<RD2));
		

	//NIC FIFO noch nicht leer
	//gehe zu Routine GET und lese ein weiteres Packet
	while (ByteL != ByteH) //(!= bedeutet ungleich)
	{
		Ethernet_Receive();
		
		// disable interrupts so that the following operations are not
		// interupted until Data is read
		cli();
		//auslesen des Empfangsbuffer BNRY = CURR 
		ByteL = ReadRTL(BNRY); //auslesen NIC Register bnry
		WriteRTL ( CR ,(1<<STA|1<<RD2|1<<PS0));
		
		ByteH = ReadRTL(CURR); //auslesen NIC Register curr
		WriteRTL ( CR ,(1<<STA|1<<RD2));
	} 

	
*/	//Zuruecksetzen der Interrupt Bits des NIC
	WriteRTL (ISR, (1<<PRX|1<<PTX|1<<RXE|1<<TXE|1<<OVW|1<<CNT|1<<RDC|1<<RST));
	
	//startet die Network Card
	Networkcard_Start();

	
/*	#if DEBUG_ETHERNET
		for (unsigned long ctr = 0; ctr < 80000; ctr++) nop();
	#endif
*/

	//Globale wieder Interrupts Einschalten
	sei ();
	#if DEBUG_CALLS
		print("\n");
		printf("Interrupt ENDE %i!!</Interrupt>\n",c);
	#endif
}
