/*#######################################################################################

AVR Embedded Webservice Project

File        : ip.c

Description : Implementation of the IP protocol.

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#include "main.h"

// IP_BlockReceive == 1 disables ability to receive IP packets
char IP_BlockReceive=0;

// IP_BlockSend == 1 disables ability to send IP packets. If receiving and
// sending IP Buffer are the same both IP_BlockSend points to &IP_BlockReceive.
char *IP_BlockSend = &IP_BlockReceive;

int ip_id_counter =0;

//############################################################################
signed char IP_ReadHeader(struct IP_HEADER* buffer, int *bufferlen){
//############################################################################
	unsigned long sum=0;
	unsigned int i;

	Read_Ethernet_Byte((unsigned char*)buffer, IP_HEADER_SIZE);
	(*bufferlen)-=IP_HEADER_SIZE;
	//checksum
	for (i=0;i<IP_HEADER_SIZE;i=i+2){
		sum +=((((unsigned long)(((unsigned char*)buffer)[i])<<8)&0x0000FF00)+
		              (((unsigned char*)buffer)[i+1]&0x000000FF));
	}
	#if DEBUG_IP
//		debug_print("Header without Options:", (unsigned char *)buffer,(unsigned char *)bufferlen);
	#endif
	#if DEBUG_IP
/*		printf("  Version: %d\n", buffer->version_ihl >>4);
		printf("  IHL:     %d\n", buffer->version_ihl & 0x0f);*/
	#endif
	
	// check for options
	if((buffer->version_ihl & 0x0f)>IP_HEADER_SIZE / 4){
		// there are options, read and discard them, we don't route anyways
		#if DEBUG_IP
			printf("There are options...");
		#endif
		unsigned char tmp[2];
		i=0;
		do{
			Read_Ethernet_Byte(&(tmp[i % 2]), 1);
			if(i % 2 == 1){
				sum+=(long)(((tmp[0]<<8)&0xFF00)+
				             (tmp[1]&0xFF));  
			}
			i++;
			(*bufferlen)--;
		// read options until we reach ihl, or if ihl=0x0f until the end of options
		} while((
					   (i<(buffer->version_ihl & 0x0f)) 
					&& ((buffer->version_ihl & 0x0f)<0x0f)
		        ) || (
		        	tmp[(i-1)%2]==0x00)
		        );
		buffer->version_ihl=buffer->version_ihl & (0xf0 + IP_HEADER_SIZE / 4);		
		if(i % 2==0) {
				sum +=(long)(((tmp[0]<<8)&0xFF00));
		}
		#if DEBUG_IP
			printf("which were discarded");
		#endif
	}	
	// take only 16 bits out of the 32 bit sum and add up the carries
	sum = (sum & 0x0000FFFF)+(sum >> 16);
	sum = (sum & 0x0000FFFF)+(sum >> 16);

	// one's complement the result
	sum = ~sum;
	#if DEBUG_IP
		printf("  check    %d\n", (int)sum);
	#endif
	
	// if checksum!=sum return -1
	if ((int)sum==0x0000) return 0;
	else return -1;
}

/*
unsigned int ip_sum_calc(unsigned int len_ip_header, unsigned int buff[]){
	unsigned int word16;
	unsigned long sum=0;
	unsigned int i;
    
	// make 16 bit words out of every two adjacent 8 bit words in the packet
	// and add them up
	for (i=0;i<len_ip_header;i=i+2){
		word16 =((buff[i]<<8)&0xFF00)+(buff[i+1]&0xFF);
		sum = sum + (u32) word16;	
	}
	
	// take only 16 bits out of the 32 bit sum and add up the carries
	while (sum>>16)
	  sum = (sum & 0xFFFF)+(sum >> 16);

	// one's complement the result
	sum = ~sum;
	
return ((u16) sum);
}
*/

//############################################################################
signed char IP_Receive(unsigned char* packet,int packetlen){
//############################################################################
	
	#if DEBUG_CALLS
		print("<IP_RECEIVE>\n");
	#endif
	int leftbuffer=packetlen;
	
	// check if IP_RECEIVE is blocked
	if(IP_BlockReceive!=0){ 
		#if DEBUG_CALLS || DEBUG_TCP
			print("IP Receive blocked\n");
		#endif
		Read_Ethernet_Terminate(packetlen);
		return 0;	
	}
	
	// If Packet>MTU we cannot handle it > Drop it
	if (packetlen>MTU){
		#if DEBUG_CALLS
			print("Packet to large\n");
			printf("MTU: %i, Packet: %i\n",MTU, packetlen);
		#endif
		Read_Ethernet_Terminate(packetlen);
		return -1;	
	}
	
	IP_BlockReceive=1;
	struct IP_HEADER *ip;
	ip = (struct IP_HEADER *) packet;
	#if DEBUG_IP
		print("reading IP Header...\n");
	#endif
	
	// Read IP Header. This function will fail, if Header Checksum was not
	// correct. Any options which are part of the header are discarded
	if(IP_ReadHeader(ip,&leftbuffer)!=0){
		#if DEBUG_IP
			print("ReadHeader failed\n");
		#endif
		IP_RECEIVE_FAIL();
	}
	#if DEBUG_IP
		print("Header OK\n");
	#endif

	#if DEBUG_IP
		printf("  total length..:0x%04x\n",
			(ip->total_length[0]<<8)+ip->total_length[1]);
	#endif
	#if DEBUG_IP && DEBUG_VERBOSE
		printf("  (bufferlen....:0x%04x)\n",packetlen);
		printf("  (still to read:0x%04x)\n",leftbuffer);
	#endif

	// check for correct version
	if(ip->version_ihl >> 4!=0x04){
		IP_RECEIVE_FAIL();
	}
	// drop packets with fragmentation
	if(((ip->fragment_stuff[0] & 0x1E)!=0)
	 ||((ip->fragment_stuff[1] & 0xFF)!=0)){
		IP_RECEIVE_FAIL();
	}
	// TTL doesn't matter to us because we don't forward
	// --- do nothing here
	
	// check IP Address (are we target?)
	if((!IP_COMPARE(ip->destAddr,ipconfig->ipAddress)
	  && (!(IP_IS_BROADCAST(ip->destAddr))))){
		IP_RECEIVE_FAIL();
	}
	#if DEBUG_IP
		printf("  Identification......%i\n",(ip->id[0]<<8)+ip->id[1]);
	#endif
	// Header ok, lets read the remaining part of the packet
	Read_Ethernet_Byte(&((unsigned char *)ip)[IP_HEADER_SIZE], leftbuffer);
	Read_Ethernet_Terminate(0);
	
	// Switch global Interrupts back on. IP is still blocked though.
	// Update: Lets leave Interrupt logics to application, no unblock here
//	sei();
//	WriteRTL (ISR,(1<<PRX|1<<PTX|1<<RXE|1<<TXE|1<<OVW|1<<CNT|1<<RDC|1<<RST));
	
	// It seems we got a valid IP packet, lets check the type and pass
	// it to the data handler
	switch (ip->protocol){
		// ICMP
		case ICMP_PROTOCOL	: 
			ICMP_Receive(
					&ip->destAddr,
					&ip->sourceAddr,
					&((unsigned char *)ip)[IP_HEADER_SIZE], 
					leftbuffer-(packetlen-CHAR2INT(ip->total_length))
				);
		break;
		// TCP
		case TCP_PROTOCOL :
			TCP_Receive(
					&ip->destAddr,
					&ip->sourceAddr,
					&((unsigned char *)ip)[IP_HEADER_SIZE], 
					leftbuffer-(packetlen-CHAR2INT(ip->total_length))
				);
		break;
		
		// UDP
		case UDP_PROTOCOL	:
			UDP_Receive(
					&ip->destAddr,
					&ip->sourceAddr,
					&((unsigned char *)ip)[IP_HEADER_SIZE],
					leftbuffer-(packetlen-CHAR2INT(ip->total_length))
				);
		break;
			
	}
	
	IP_BlockReceive = 0;		
		
	#if DEBUG_CALLS
		print("</IP_RECEIVE>\n");
	#endif
	return 0;
}

//############################################################################
int IP_Send(unsigned char *dstip, unsigned char *srcip, 
                    unsigned char *packet, int packetlen,
                    unsigned char protocol, unsigned char DF, unsigned char ttl){
//############################################################################

	// IP_BlockSend >= 2 whenever another IP_Send Routine is active. This
	// mechanism allows IP_Send to block IP_Receive also, in case 
	// IP_BlockSend == &IP_BlockReceive, but IP_Receive won't block IP_Send.
	// This is OK, knowing that any application receiving IP Data has to
	// process it completely before sending. This is a limitation of the one
	// Buffer for IP_Send and IP_Receive architecture.
	if (*IP_BlockSend >= 2) {
		return -1;
	}
	
	(*IP_BlockSend)+=2;

	#if DEBUG_CALLS
		print("<IP_SEND>\n");
  		printf("receiving pointers: %i(%02x%02x%02x%02x), %i(%02x%02x%02x%02x), %i",
  			(unsigned int)dstip,dstip[0],dstip[1],dstip[2],dstip[3],
  			(unsigned int)srcip,srcip[0],srcip[1],srcip[2],srcip[3],
  			(unsigned int)packet);
	#endif
	long sum = 0;
	int i;
	// allocate send buffer
	struct IP_HEADER* ip = 
		(struct IP_HEADER*)&(reservedMemory[MEM_IP_SEND_OFFSET]);
	unsigned char * dstMac;
	unsigned char type[2] = IP_ETHERNET_TYPE;
	
//	extern unsigned char time_short;

	#if DEBUG_IP
		print("building IP Header...\n");
	#endif
	// fill IP Header
	//                           IHL: number of 32bit words in ip header
	ip->version_ihl = (0x4<<4) + IP_HEADER_SIZE/4;
	ip->tos = 0;
	ip->total_length[0] = (IP_HEADER_SIZE+packetlen)>>8;
	ip->total_length[1] = 0x00FF & (IP_HEADER_SIZE+packetlen);
	ip->id[0]=(++ip_id_counter)>>8;
	ip->id[1]=(++ip_id_counter)&0x00FF;
	// set "don't fragment"-flag
	ip->fragment_stuff[0]=DF<<6;
	ip->fragment_stuff[1]=0;
	ip->ttl=ttl;
	ip->protocol=protocol;

	// set Source Addr
	#if DEBUG_IP
		printf("set SrcAddr:%i.%i.%i.%i\n",srcip[0],srcip[1],srcip[2],srcip[3]);
	#endif
	ip->sourceAddr[0]=srcip[0];
	ip->sourceAddr[1]=srcip[1];
	ip->sourceAddr[2]=srcip[2];
	ip->sourceAddr[3]=srcip[3];
	// set Dest Addr
	#if DEBUG_IP
		printf("set DstAddr:%i.%i.%i.%i\n",dstip[0],dstip[1],dstip[2],dstip[3]);
	#endif
	ip->destAddr[0]=dstip[0];
	ip->destAddr[1]=dstip[1];
	ip->destAddr[2]=dstip[2];
	ip->destAddr[3]=dstip[3];

	// make sure header checksum is 0 before summing up
	ip->header_checksum[0]=0x00;
	ip->header_checksum[1]=0x00;
	// calculate header checksum
	#if DEBUG_IP
		print("  calculate checksum...");
	#endif
	for (i=0;i<IP_HEADER_SIZE;i=i+2){
		sum +=(((unsigned long)((((unsigned char*)ip)[i])<<8)&0x0000FF00)+
		              (((unsigned char*)ip)[i+1]&0x000000FF));
	}
	// take only 16 bits out of the 32 bit sum and add up the carries
	sum = (sum & 0x0000FFFF)+(sum >> 16);
	sum = (sum & 0x0000FFFF)+(sum >> 16);
	sum = ~sum;
	#if DEBUG_IP
		printf("0x%04x\n",(unsigned int)sum);
	#endif
	// set checksum
	ip->header_checksum[0]=(sum>>8);
	ip->header_checksum[1]=sum & 0x00FF;
		
	#if DEBUG_IP
		printf("set %i byte Payload: ",packetlen);
	#endif
	// copy Payload into Send Buffer
	for (i=0;i<packetlen;i++){
		((unsigned char*)ip)[IP_HEADER_SIZE+i] = packet[i];
		#if DEBUG_IP && DEBUG_VERBOSE
			printf("%02x",((unsigned char*)ip)[IP_HEADER_SIZE+i]);
		#endif
	}
	#if DEBUG_IP
		print("\n");
	#endif
	
	#if DEBUG_IP
		print("check target in Subnet: ");
	#endif
	
	// check if destination is broadcast address
	if (
	  (dstip[0]==255)
	&&(dstip[1]==255)
	&&(dstip[2]==255)
	&&(dstip[3]==255)
	) {
		#if DEBUG_IP
			print("Broadcast: ");
		#endif
		dstMac = (unsigned char *)&mac_broadcast_addr;
		
	// check if destination is in local subnet
	} else if (
/*	  (ipconfig->ipAddress[0]&ipconfig->netMask[0])==(dstip[0]&ipconfig->netMask[0])
	&&(ipconfig->ipAddress[1]&ipconfig->netMask[1])==(dstip[1]&ipconfig->netMask[1])
	&&(ipconfig->ipAddress[2]&ipconfig->netMask[2])==(dstip[2]&ipconfig->netMask[2])
	&&(ipconfig->ipAddress[3]&ipconfig->netMask[3])==(dstip[3]&ipconfig->netMask[3])*/
	  (CHAR2LONG(ipconfig->ipAddress)&CHAR2LONG(ipconfig->netMask))
	   == (CHAR2LONG(dstip)&CHAR2LONG(ipconfig->netMask))
	) {
		// get destination MAC address
		#if DEBUG_IP
			printf("IP Address Match: %02x&%02x=%02x",
				ipconfig->ipAddress[0],ipconfig->netMask[0],
				(unsigned char)((CHAR2LONG(ipconfig->ipAddress)&CHAR2LONG(ipconfig->netMask))>>24)&0xFF);
			print("send directly to Dst: ");
		#endif
		
		dstMac = NULL;
//		unsigned char time_now = time_short;
		unsigned char retry = 0;
		
//		printf("%i.%i.%i.%i\n",dstip[0],dstip[1],dstip[2],dstip[3]);
	  	#if DEBUG_CALLS
	  		printf("ARP_Cache_Check with pointer: %i\n",(int)dstip);
	  	#endif
		dstMac = ARP_Cache_Check(dstip);
		while((dstMac == NULL) && (retry < ARP_RETRY_MAX)){
		 	// We found no matching entry, return NULL.
		  	#if DEBUG_CALLS
		  		printf("ARP_Request with pointer: %i\n",(int)dstip);
		  	#endif
			ARP_Request(dstip);
			sei();
			for (i=0;i<10000&&dstMac==NULL;i++){
				dstMac = ARP_Cache_Check(dstip);
			}
//			while((time_now == time_short) && (dstMac == NULL)) ;
//			time_now = time_short;
			retry++;
		}
	} else {
		// get destination MAC address
		#if DEBUG_IP
			print("send to Gateway: ");
		#endif
		// get gateway MAC address
		
		dstMac = NULL;
//		unsigned char time_now = time_short;
		unsigned char retry = 0;
		dstMac = ARP_Cache_Check((unsigned char*)&(ipconfig->gateway));
		while((dstMac == NULL) && (retry < ARP_RETRY_MAX)){
			retry++;
			#if DEBUG_IP
				printf("ARP Retry: %i\n",retry);			
			#endif
		 	// We found no matching entry, return NULL.
			ARP_Request((unsigned char*)&(ipconfig->gateway));
			sei();
			for (i=0;i<10000&&dstMac==NULL;i++){
				dstMac = ARP_Cache_Check((unsigned char*)&(ipconfig->gateway));
			}
//			while((time_now == time_short) && (dstMac == NULL)) ;
//			time_now = time_short;
		}
	}
/*
//	Stefans Home Rechner	
//	char tmpDstMac[6] = {0x00,0x10,0xb5,0xce,0xf2,0xc6};

//	Jons Laptop
	char tmpDstMac[6] = {0x08,0x00,0x46,0xB6,0x62,0x64};

	dstMac = (char *)tmpDstMac;
*/
	// Instead of aborting, one could also wait in a loop for some time and
	// retry ARP, because a ARP reply might have gotten in
	if (dstMac==NULL) {
		// get destination MAC address
		#if DEBUG_IP
			print("ARP failed, abort!\n");
		#endif
		(*IP_BlockSend)-=2;
		return -1;
	}
	
	// get destination MAC address
	#if DEBUG_IP
		printf("MAC Address: %02x:%02x:%02x:%02x:%02x:%02x\n",
			dstMac[0],dstMac[1],dstMac[2],dstMac[3],dstMac[4],dstMac[5]);
	#endif
	// Send Ethernet Packet
	Ethernet_Send (dstMac, type, (char *)ip, packetlen+IP_HEADER_SIZE);	
	
	#if DEBUG_CALLS
		print("</IP_SEND>\n");
	#endif
	(*IP_BlockSend)-=2;
	return packetlen;
}

//############################################################################
signed char ICMP_Receive(IP_ADDRESS *dstip, IP_ADDRESS *srcip, 
                         unsigned char *packet, int packetlen){
//############################################################################
	unsigned long sum=0;
	unsigned int i;

	#if DEBUG_CALLS
		print("-- ICMP_RECEIVE\n");
	#endif
	// control Checksum
	for (i=0;i<packetlen-1;i=i+2){
		sum +=(((unsigned long)((packet[i])<<8)&0x0000FF00)+
		              (packet[i+1]&0x000000FF));
	}
	// if packet length is odd a 0-byte has to be assumed at the end
	if (packetlen%2==1){
		sum +=((unsigned long)((packet[packetlen-1])<<8)&0x0000FF00);
	}
	// take only 16 bits out of the 32 bit sum and add up the carries
	sum = (sum & 0x0000FFFF)+(sum >> 16);
	sum = (sum & 0x0000FFFF)+(sum >> 16);
	// sum has to be 0 to be valid, otherwise fail
	if ((int)(~sum) != 0x0000) {
		#if DEBUG_ICMP
			print("invalid ICMP Packet (checksum failed)\n");
		#endif
		return -1;
	}
	
	// checksum seems to be valid, lets analyse the ICMP header
	struct ICMP_HEADER* icmp_h = (struct ICMP_HEADER *)packet;
	switch (icmp_h->type){
		// Echo (0x8) requests need to be replied
		case 0x08:
			#if DEBUG_ICMP
				print("ICMP ECHO Request received... constructing reply\n");
			#endif
				// Reply can be constructed in the same buffer
				icmp_h->type=0x00;
				// Reset checksum
				icmp_h->checksum[0]=0x00;
				icmp_h->checksum[1]=0x00;
				// calc checksum
				sum = 0;
				for (i=0;i<packetlen-1;i=i+2){
					sum +=(((unsigned long)((packet[i])<<8)&0x0000FF00)+
					              (packet[i+1]&0x000000FF));
				}
				// if packet length is odd a 0-byte has to be assumed at the end
				if (packetlen%2==1){
					sum +=((unsigned long)((packet[packetlen-1])<<8)&0x0000FF00);
				}
				// take only 16 bits out of the 32 bit sum and add up the carries
				sum = (sum & 0x0000FFFF)+(sum >> 16);
				sum = (sum & 0x0000FFFF)+(sum >> 16);
				sum = ~sum;
				// set checksum
				icmp_h->checksum[0]=sum>>8;
				icmp_h->checksum[1]=sum & 0x00FF;
				// Send 
				IP_ADDRESS newdst = {0,0,0,0};
				newdst[0] = (*srcip)[0];
				newdst[1] = (*srcip)[1];
				newdst[2] = (*srcip)[2];
				newdst[3] = (*srcip)[3];
				
				return IP_Send((char *)newdst, (char *)ipconfig->ipAddress, packet, packetlen,
                    0x01, 1, 60);

		break;
		default:
			#if DEBUG_ICMP
				print("unknown ICMP request\n");
			#endif
			return -1;
		break;
	}

	return -1;		
}
