/*#######################################################################################

AVR Embedded Webservice Project

File        : ip.h

Description : Implementation of the IP protocol (header).

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#ifndef _IP_H_
#define _IP_H_

#define IP_HEADER_SIZE       20
#define IP_COMPARE(x,y)      x[0]==y[0]&&x[1]==y[1]&&x[2]==y[2]&&x[3]==y[3]
#define IP_IS_BROADCAST(x)   x[0]==255&&x[1]==255&&x[2]==255&&x[3]==255
#if DEBUG_CALLS
	#define IP_RECEIVE_FAIL()    Read_Ethernet_Terminate(leftbuffer);IP_BlockReceive = 0;printf("</IP_RECEIVE>\n");return -1;
#else
	#define IP_RECEIVE_FAIL()    Read_Ethernet_Terminate(leftbuffer);IP_BlockReceive = 0;return -1;
#endif
#define IP_ETHERNET_TYPE	 {0x08,0x00}
#define ICMP_PROTOCOL        0x01
#define ARP_RETRY_MAX        5


typedef unsigned char IP_ADDRESS[4];

typedef struct IP_HEADER{
	unsigned char version_ihl;// je 4 bit
	unsigned char tos;// type of servive
	unsigned char total_length[2];
	unsigned char id[2];
	unsigned char fragment_stuff[2];
	unsigned char ttl;
	unsigned char protocol;
	unsigned char header_checksum[2];
	IP_ADDRESS sourceAddr;
	IP_ADDRESS destAddr;	
} T_IPHEADER;

typedef struct ICMP_HEADER{
	unsigned char type;																		
	unsigned char code;
	unsigned char checksum[2];
	unsigned char id[2];
	unsigned char seq[2];
} T_ICMPHEADER;

struct IPCONFIG {
	IP_ADDRESS ipAddress;
	IP_ADDRESS netMask;
	IP_ADDRESS gateway;
};

extern int ip_id_counter;
extern char IP_BlockReceive;

extern struct IPCONFIG* ipconfig;

//extern struct IPCONFIG* ipconfig;
int IP_Send(unsigned char *dstip, unsigned char *srcip, 
                    unsigned char *packet, int packetlen,
                    unsigned char protocol, unsigned char DF, unsigned char ttl);
extern signed char IP_Receive(unsigned char *packet,int packetlen);
extern signed char ICMP_Receive(IP_ADDRESS *dstip, IP_ADDRESS *srcip, 
                         unsigned char *packet, int packetlen);


#endif //_IP_H_
