/*#######################################################################################

AVR Embedded Webservice Project

File        : mem.h

Description : Global declarations of memory regions (header).

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#ifndef _MEM_H_
#define _MEM_H_

#include "ip.h"
#include "tcp.h"
#include "ethernet.h"
#include "arp.h"
#include "dhcp.h"
#include "udp.h"

//Daten Array. Hier sind alle globalen Puffer enthalten. Enthaltene Puffer sind:
// EthernetPuffer (ETHER_HEADER_SIZE)
// ARP Puffer     (ARP_SIZE)
// IP Puffer      (MTU)
// TCP Puffer     (TCP_SIZE)
// IP Config
// ... was da dann alles maximal parallel noch rein muss.
// wichtig: Es muss nur so viel Speicher reserviert werden, wie maximal zu einer
// Zeit genutzt wird.

// All Applications' Buffer needs are to be defined here
#define DHCP_SEND_SIZE      UDP_HEADER_SIZE+DHCP_SIZE
#define DHCP_RECEIVE_SIZE   DHCP_SEND_SIZE
// #define HTTP_RECEIVE_SIZE
// #define HTTP_SEND_SIZE

// MAX_APP_SIZE defines the maximum memory an application needs. This should
// be the maximum of all running applications' needs.
//#define MAX_APP_SIZE    DHCP_SEND_SIZE
#define MAX_APP_SIZE	400
#define MTU             IP_HEADER_SIZE+MAX_APP_SIZE
//#define MEM_CONN        UDP_MAX_SOCKETS*sizeof(struct UDP_SOCKET)
#define RESERVED_MEMORY ETHER_HEADER_SIZE+ARP_SIZE+MTU

unsigned char reservedMemory[RESERVED_MEMORY];
struct TCP_SOCKET tcp_sockets[TCP_MAX_SOCKETS];
struct TCB tcp_connections[TCP_MAX_CONNECTIONS];
struct IPCONFIG ipconfig_mem;
unsigned char TCP_Retransmission_Buffer[TCP_PAYLOAD_SIZE];
struct ARP_Cache arp_cache[ARP_CACHE_SIZE];
extern struct UDP_SOCKET udp_sockets[];
extern struct DHCP_Config dhcp_config;

// Es gibt einige Puffer, die direkt auf den Speicher zugreifen m???ssen, z.B.
// Ethernet bei einem einkommenden Paket. Hierf???r muss das Offset definiert
// werden.
#define MEM_ETHER_OFFSET       0x00
#define MEM_ARP_OFFSET         MEM_ETHER_OFFSET+ETHER_HEADER_SIZE
#define MEM_IP_RECEIVE_OFFSET  MEM_ARP_OFFSET+ARP_SIZE
#define MEM_IP_SEND_OFFSET     MEM_IP_RECEIVE_OFFSET
#define MEM_UDP_SEND_OFFSET    MEM_IP_SEND_OFFSET+IP_HEADER_SIZE
#define MEM_DHCP_OFFSET        MEM_UDP_SEND_OFFSET+UDP_HEADER_SIZE
#define MEM_TCP_SEND_OFFSET    MEM_UDP_SEND_OFFSET
//#define MEM_UDP_SOCKETS_OFFSET MEM_IP_SEND_OFFSET + MTU

extern void init_mem(void);



#endif //_MEM_H_
