/*#######################################################################################

AVR Embedded Webservice Project

File        : rtl8019.c

Description : Communication with the NIC (header).

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de
based on code from Ulrich Radig for the AVR Small Webserver 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#include "main.h"
#include "rtl8019.h"

//Routinen zum ansteuern der Netzwerkkarte 

//############################################################################
// Schreibt ein Byte in ein angegebenes Register der Netzwerkkarte
void WriteRTL (unsigned char rtl_addr,unsigned char rtl_data)
//############################################################################
{
	//Der Datenport wird auf output gesetzt vom IC zum NIC schreiben
	DATA_CTRL_RLT = OUTPUT;
	//write Adress to Port
	ADDR_PORT_RLT = rtl_addr;
	//write Data to Port
	DATA_PORT_RLT_WRITE = rtl_data;
	//set write Pin to Low
	RTL_WR_OFF();
	//wait a short time
        nop(); //gib mal ein bisschen GAS Ulrich ;)
        nop();

	//set write Pin to High
	RTL_WR_ON();	
	//write to RTL complete
}


//############################################################################
// Auslesen eines Byte von einen angegebenes Register der Netzwerkkarte
unsigned char ReadRTL (unsigned char rtl_addr)
//############################################################################
{
	unsigned char rtl_data;
	//Der Datenport wir auf input gesetzt vom NIC zum IC lesen
	DATA_CTRL_RLT = INPUT;
	//write Adress to Port
	ADDR_PORT_RLT = rtl_addr;
	//set read Pin to Low
	RTL_RD_OFF();
	//wait a short time	
        nop();
        nop();

	//read from port rtldata
	rtl_data = DATA_PORT_RLT_READ;	
	//set Read Pin to High
	RTL_RD_ON();	
	//read from RTL complete
	return (rtl_data);
}

//############################################################################
//Hier wird die Netzwerkkarte Grundinizialisiert
void Init_Realtek_Network_Card (void)
//############################################################################
{

	#if defined (__AVR_ATmega128__)
		//Set Addr Port Direction = Output
		DATA_ADDR_RLT = OUTPUT;
	#endif

	#if defined (__AVR_ATmega32__)
		//Set Addr Port Direction = Output
		DATA_ADDR_RLT = OUTPUT;
	#endif

	//Set all Addrport High
	ADDR_PORT_RLT = OUTPUT;
	//set Data Port Direction = Input
	DATA_CTRL_RLT = INPUT;

	print ("Init Network Card: ");

	//Set Reset Pin Low
	RTL_RESET_OFF();		

	//added by holgi
	//meine Karten laufen nur wenn man hier noch ca. 100ms wartet
	//wait a long time
	unsigned long i = 0;
	while (i < 60000 * 5) i++;
	//end added by holgi
	
	ReadRTL (RSTPORT);
	WriteRTL (RSTPORT , 0xFF);
	
	//wait a short time
	int a = 0;
	while (a < 1000)
	{
	a++;
	}
	
	print (".");			//Point 1
	WriteRTL (CR , (1<<STP| 1<<RD2));

	//wait a short time
	a = 0;
	while (a < 1000)
	{
	a++;
	}
	
	print (".");			//Point 2
	WriteRTL (DCR , DCRVAL);
	WriteRTL (RBCR0 , 0x00);
	WriteRTL (RBCR1 , 0x00);
	WriteRTL (RCR , 0x04);
	WriteRTL (TPSR , RXSTART);
	WriteRTL (TCR , 0x02);
	WriteRTL (PSTART , RXSTART);
	WriteRTL (BNRY , RXSTART);
	WriteRTL (PSTOP , RXSTOP);
	WriteRTL (CR ,(1<<STP| 1<<RD2|1<<PS0));

	//wait a schort time	
	for (int a = 0; a < 1000; a++)
		{
		}

	print (".");			//Point 3
	WriteRTL (CURR , RXSTART);
	
	for(int a = 0; a<=5; a++)
		{
		WriteRTL ((PAR0 + a) , mymac[a]);
		print (".");		//Point 4,5,6,7,8,9
		}

	WriteRTL (CR , (1<<STP| 1<<RD2));
	WriteRTL (DCR , DCRVAL);
	WriteRTL (CR , (1<<STA| 1<<RD2));
	WriteRTL (ISR ,(1<<PRX|1<<PTX|1<<RXE|1<<TXE|1<<OVW|1<<CNT|1<<RDC|1<<RST));
	WriteRTL (IMR , IMRVAL);
	WriteRTL (TCR , TCRVAL);
	
	//startet die Network Card
	WriteRTL (CR ,(1<<STA| 1<<RD2));

	print (" Init Ready! \n");
}


//############################################################################
void Write_Ethernet_Init (unsigned int Packet_Length)
//############################################################################
{
	//Globale Interrupts Ausschalten
//	cli();
	#if DEBUG_CALLS
		print("---- Write_Ethernet_Init--------------------------\n");
	#endif
	
	if (Packet_Length < ETHER_MIN_SIZE) Packet_Length = ETHER_MIN_SIZE;

	WriteRTL (CR,(1<<STA| 1<<RD2));
	WriteRTL (TPSR,TXSTART);
	WriteRTL (RSAR0,0x00);
	WriteRTL (RSAR1,0x40);
//	WriteRTL (ISR,(1<<PRX|1<<PTX|1<<RXE|1<<TXE|1<<OVW|1<<CNT|1<<RDC|1<<RST));
	WriteRTL (RBCR0,(Packet_Length & 0x00ff));
	WriteRTL (RBCR1,(Packet_Length & 0xff00) >>8);
	WriteRTL (CR,(1<<STA| 1<<RD1));
	
	//Der Datenport wir auf output gesetzt vom IC zum NIC schreiben
	DATA_CTRL_RLT = OUTPUT;
	//write Adress to Port
	ADDR_PORT_RLT = RDMAPORT;
	
	#if DEBUG_NIC
		printf("Ready to write to %d Bytes to NIC:\n", Packet_Length);
	#endif
	
}

//############################################################################
void Write_Ethernet_Buffer (unsigned char *buffer, unsigned int bufferlen)
//############################################################################
{
	unsigned int ctr;
	
	#if DEBUG_NIC
		printf("writing %d Bytes: ",bufferlen);
	#endif
	for (ctr = 0; ctr < bufferlen; ctr++)
	{
		//write Data to Port
		DATA_PORT_RLT_WRITE = buffer[ctr];
		#if DEBUG_NIC && DEBUG_VERBOSE
			printf("%02x", buffer[ctr]);
		#endif
		//set write Pin to Low
		RTL_WR_OFF();
		//wait a short time
        	nop(); //gib mal ein bisschen GAS Ulrich ;) Holgi
        	nop();

		//set write Pin to High
		RTL_WR_ON();	
		//write to RTL complete
	}
	#if DEBUG_NIC
		print("\n");
	#endif
}	

//############################################################################
void Write_Ethernet_Terminate (unsigned int bufferlen)
//############################################################################
{
	#if DEBUG_CALLS
		print("---- Write_Ethernet_Terminate --------------------\n");
	#endif
	
	#if DEBUG_NIC
		printf("Padding: %d Bytes",
		        bufferlen<ETHER_MIN_SIZE ? ETHER_MIN_SIZE-bufferlen : 0);
	#endif
	
	unsigned char ctr;
	
	for (ctr = bufferlen; ctr < ETHER_MIN_SIZE; ctr++){
		//write Data to Port
		DATA_PORT_RLT_WRITE = 0x00;
		#if DEBUG_NIC && DEBUG_VERBOSE
			printf("__");
		#endif
		//set write Pin to Low
		RTL_WR_OFF();
		//wait a short time
		nop(); //gib mal ein bisschen GAS Ulrich ;) Holgi
		nop();
		
		//set write Pin to High
		RTL_WR_ON();	
		//write to RTL complete		
	}
		 
	#if DEBUG_NIC
		print(".\n");
	#endif

	//loop_until_bit_is_set(ReadRTL(isr),rdc);
	while ((ReadRTL(ISR)&(1<<RDC)) == 0) {}
	
	if (bufferlen<ETHER_MIN_SIZE) bufferlen = ETHER_MIN_SIZE;

	WriteRTL (TBCR0,(bufferlen & 0x00ff)/* - 4*/);
	WriteRTL (TBCR1,(bufferlen & 0xff00) >>8);
	WriteRTL (CR,(1<<TXP| 1<<RD2));
	


	//Globale Interrupts einschalten
//	sei();
}

//############################################################################
int Read_Ethernet_Init (void)
//############################################################################
{
	#if DEBUG_CALLS
		print("---- Read_Ethernet_Init --------------------------\n");
	#endif
	unsigned int tmp;
	
	WriteRTL(CR ,(1<<STA| 1<<RD0| 1<<RD1));	
	ReadRTL(RDMAPORT);
	ReadRTL(RDMAPORT);
	tmp = ReadRTL(RDMAPORT);
	return ((ReadRTL(RDMAPORT)<<8)+tmp-ETHER_TRAILER_SIZE);
}

//############################################################################
void Read_Ethernet_Byte (char *buffer, unsigned int count)
//############################################################################
{
	#if DEBUG_NIC
		printf("Read %d Bytes from Network Interface\n",count);
	#endif

	unsigned int ctr;
	
	//Der Datenport wir auf input gesetzt vom NIC zum IC lesen
	DATA_CTRL_RLT = INPUT;
	//write Adress to Port
	ADDR_PORT_RLT = RDMAPORT;
	
	//Daten in den Buffer uebernehmen
	for(ctr=0; ctr<count; ctr++)
	{
		//set read Pin to Low
		RTL_RD_OFF();
		//wait a short time	
		nop();
		nop();
		//read from port rtldata
		buffer[ctr] = DATA_PORT_RLT_READ;	
		//set Read Pin to High
		RTL_RD_ON();	
		//read from RTL complete
	}
}

//############################################################################
void Read_Ethernet_Terminate (unsigned int count)
//############################################################################

{
	#if DEBUG_CALLS
		print("---- Read_Ethernet_Terminate ---------------------\n");
	#endif
	unsigned int ctr;
	unsigned char tmp = 0;
	#if DEBUG_NIC
		printf("Drop remaining %d Bytes and terminate reading.\n",count);
	#endif
	
	for(ctr=0; ctr<(count+ETHER_TRAILER_SIZE); ctr++)
	{

		//set read Pin to Low
		RTL_RD_OFF();
		//wait a short time	
		nop();
		nop();
		//read from port rtldata
		tmp = DATA_PORT_RLT_READ;	
		//set Read Pin to High
		RTL_RD_ON();	
		//read from RTL complete

		#if DEBUG_NIC && DEBUG_VERBOSE
			printf("%02x", tmp);
		#endif

	}
	#if DEBUG_NIC && DEBUG_VERBOSE
		print("\n");
	#endif
	
	tmp = tmp & RDC;
	
	if ( tmp != 64 )
	{
		ReadRTL(ISR);
	}

}

