/*#######################################################################################

AVR Embedded Webservice Project

File        : rtl8019.h

Description : Communication with the NIC (header).

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de
based on code from Ulrich Radig for the AVR Small Webserver 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#ifndef _RTL_8019_H
 #define _RTL_8019_H

//Prototype
extern void WriteRTL (unsigned char,unsigned char);
extern unsigned char ReadRTL (unsigned char);
extern void Init_Realtek_Network_Card (void);
extern void Write_Ethernet_Init (unsigned int Packet_Length);
extern void Write_Ethernet_Buffer (unsigned char *buffer, unsigned int bufferlen);
extern void Write_Ethernet_Terminate (unsigned int bufferlen);
extern int Read_Ethernet_Init (void);
extern void Read_Ethernet_Terminate (unsigned int count);
extern void Read_Ethernet_Byte (char *buffer, unsigned int count);

#define nop()  __asm__ __volatile__ ("nop" ::)

#define RTL_WR_OFF() cbi(ADDR_PORT_RLT,WRITE_PIN);
#define RTL_WR_ON() sbi(ADDR_PORT_RLT,WRITE_PIN);

#define RTL_RD_OFF() cbi(ADDR_PORT_RLT,READ_PIN);
#define RTL_RD_ON()  sbi(ADDR_PORT_RLT,READ_PIN);

#define RTL_RESET_OFF() cbi(ADDR_PORT_RLT,RESET_PIN);

#define Networkcard_Start() WriteRTL (CR ,(1<<STA|1<<RD2));


//Ethernet Card Ports
#define DATA_CTRL_RLT  			DDRA
#define DATA_ADDR_RLT			DDRC	//Fuer ATMega128/32 wird bei ATMega102 nicht ben.
#define ADDR_PORT_RLT  			PORTC
#define DATA_PORT_RLT_WRITE  	PORTA
#define DATA_PORT_RLT_READ 		PINA	//Read
#define WRITE_PIN				6		//gibt den Pin an WriteIO Signal angeschlossen
#define READ_PIN				5		//gibt den Pin an ReadIO Signal angeschlossen
#define RESET_PIN				7		//gibt den Pin an Reset angeschlossen ist
#define INPUT					0x00	//setzt einen Port auf Input
#define OUTPUT					0xff	//setzt einen Port auf Output

//Realtek Netzwerkkarten Register
#define RTL_REG_OFFSET				96
#define CR						RTL_REG_OFFSET+0x00
#define PSTART					RTL_REG_OFFSET+0x01
#define PAR0					RTL_REG_OFFSET+0x01                                          
#define PSTOP					RTL_REG_OFFSET+0x02
#define BNRY					RTL_REG_OFFSET+0x03
#define TPSR					RTL_REG_OFFSET+0x04
#define TBCR0					RTL_REG_OFFSET+0x05
#define TBCR1					RTL_REG_OFFSET+0x06
#define ISR						RTL_REG_OFFSET+0x07
#define CURR					RTL_REG_OFFSET+0x07                                       
#define RSAR0					RTL_REG_OFFSET+0x08
#define CRDA0					RTL_REG_OFFSET+0x08
#define RSAR1					RTL_REG_OFFSET+0x09	
#define CRDAl					RTL_REG_OFFSET+0x09
#define RBCR0					RTL_REG_OFFSET+0x0A
#define RBCR1					RTL_REG_OFFSET+0x0B
#define RSR						RTL_REG_OFFSET+0x0C
#define RCR						RTL_REG_OFFSET+0x0C
#define TCR						RTL_REG_OFFSET+0x0D
#define CNTR0					RTL_REG_OFFSET+0x0D
#define DCR						RTL_REG_OFFSET+0x0E
#define CNTR1					RTL_REG_OFFSET+0x0E
#define IMR						RTL_REG_OFFSET+0x0F
#define CNTR2					RTL_REG_OFFSET+0x0F
#define RDMAPORT				RTL_REG_OFFSET+0x10
#define RSTPORT					RTL_REG_OFFSET+0x18

//RTL8019AS initial register Werte
#define RCRVAL					0x04
#define TCRVAL					0x00
#define DCRVAL					0x58

//PXR and OVW interrupt enabled
#define IMRVAL					0x11
#define TXSTART					0x40
#define RXSTART					0x46
#define RXSTOP					0x60

//RTL8019AS ISR Register Bits
#define PRX						0	//BIT 0 0x01
#define	PTX						1	//BIT 1 0x02
#define	RXE						2	//BIT 2 0x04
#define	TXE						3	//BIT 3 0x08
#define	OVW						4	//BIT 4 0x10
#define	CNT						5	//BIT 5 0x20
#define	RDC						6	//BIT 6 0x40
#define	RST						7	//BIT 7 0x80

//RTL8019AS CR Register Bits
#define STP						0	//BIT 0 0x01
#define	STA						1	//BIT 1 0x02
#define	TXP						2	//BIT 2 0x04
#define	RD0						3	//BIT 3 0x08
#define	RD1						4	//BIT 4 0x10
#define	RD2						5	//BIT 5 0x20
#define	PS0						6	//BIT 6 0x40
#define	PS1						7	//BIT 7 0x80

#endif // _RTL_8019_H


