/*#######################################################################################

AVR Embedded Webservice Project

File        : tcp.c

Description : Implementation of the TCP protocol.

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#include "main.h"

char TCP_Receive_Data_Blocked = 0;

//############################################################################
signed char TCP_Receive(IP_ADDRESS *dstAddr, IP_ADDRESS *srcAddr, 
//############################################################################
  unsigned char* packet,int packetlen) {
	unsigned long sum = 0;
	int i;
	TCP_CONN_ID c=0, c_temp;
	TCP_SCKT_ID s;
	  	
  	#if DEBUG_CALL
  		print("<TCP_RECEIVE>");
  	#endif

	#if DEBUG_TCP	
  		printf("Receiving %i Bytes TCP Packet: ",packetlen);
  	#endif
  	
	struct TCP_HEADER * tcp = (struct TCP_HEADER *) packet;  	
  	
	// ###### C h e c k s u m ##################################################
  	// TODO: Fix Checksum Calculation
  	// check checksum

	// Pseudoheader
	sum+=(((*srcAddr)[0]<<8)&0xFF00)+((*srcAddr)[1]&0xFF);
	sum+=(((*srcAddr)[2]<<8)&0xFF00)+((*srcAddr)[3]&0xFF);
	#if (DEBUG_TCP && DEBUG_VERBOSE)
		printf("Pseudoheader:\n  SourceAddr %02x%02x+%02x%02x=%04x carry:%04x\n",
			(*srcAddr)[0],(*srcAddr)[1],(*srcAddr)[2],
			(*srcAddr)[3],(int)sum, (int)(sum>>16));
	#endif

	sum+=(((*dstAddr)[0]<<8)&0xFF00)+((*dstAddr)[1]&0xFF);
	sum+=(((*dstAddr)[2]<<8)&0xFF00)+((*dstAddr)[3]&0xFF);
	#if (DEBUG_TCP && DEBUG_VERBOSE)
		printf("  DestAddr %02x%02x+%02x%02x=%04x carry:%04x\n",(*dstAddr)[0],
			(*dstAddr)[1],(*dstAddr)[2],(*dstAddr)[3],(int)sum, (int)(sum>>16));
	#endif

	sum+=(0x00FF & TCP_PROTOCOL);	
	#if (DEBUG_TCP && DEBUG_VERBOSE)
		printf("  Protocoll %02x%02x=%04x carry:%04x\n",
			0x00,TCP_PROTOCOL,(int)sum, (int)(sum>>16));
	#endif

	sum+=packetlen & 0xFF00;
	sum+=packetlen & 0x00FF;
	#if (DEBUG_TCP && DEBUG_VERBOSE)
		printf("  Paketlen %04x\nHeader",packetlen);
	#endif
	// Packet
	for (i=0;i<packetlen-1;i=i+2){
		sum +=((((unsigned long)(((unsigned char*)packet)[i])<<8)&0x0000FF00)+
		              (((unsigned char*)packet)[i+1]&0x000000FF));

		#if (DEBUG_TCP && DEBUG_VERBOSE)
			printf("  +%02x%02x=%04x carry:%04x\n",packet[i],packet[i+1],
				(int)sum, (int)(sum>>16));
		#endif
	}
	// if packet length is odd a 0-byte has to be assumed at the end
	if (packetlen%2==1){
		sum +=((unsigned long)((packet[packetlen-1])<<8)&0x0000FF00);
		#if (DEBUG_TCP && DEBUG_VERBOSE)
			printf("  +%02x%02x=%04x carry:%04x\n",packetlen-1,0x00,
				(int) sum, (int)(sum>>16));
		#endif
	}

	// take only 16 bits out of the 32 bit sum and add up the carries
	sum = (sum & 0x0000FFFF)+(sum >> 16);
	sum = (sum & 0x0000FFFF)+(sum >> 16);

	sum = ~sum;
	  	  	
  	#if (DEBUG_TCP && DEBUG_VERBOSE)
  		printf("  TCP Checksum: 0x%02x%02x",tcp->checksum[0],tcp->checksum[1]);
  		printf("  check: 0x%04x\n",(int)sum);
  	#endif
  	  	
  	if (((int)sum!=0x0000)){
  		#if DEBUG_TCP
  			print("TCP Checksum failed... continue anyways ;-)\n");
  		#endif
  		//return -1;
  	} else {
  		#if DEBUG_TCP
  			print("TCP Checksum OK\n");
  		#endif
  	}
  	

	// ###### F I N D   M A T C H I N G   C O N N E C T I O N ##################
	// if no unack'ed connection was found c equals TCP_MAX_CONNECTIONS

	// NO connection waiting for ACKs?
	// check if local socket exists and is bound to connection
	for (i=0; i<TCP_MAX_CONNECTIONS; i++){
	  	#if DEBUG_TCP
	  		printf("Scan connection %i ... PORT: %i\n",i,
		  		tcp_sockets[tcp_connections[i].localSocket].port);
	  	#endif
		// check for matching local socket. Only if connection is bound
		// to the requested local socket the tcp request may be processed
		if (((unsigned int)(tcp_sockets[tcp_connections[i].localSocket].port&(0xFFFF)) == ((unsigned int)CHAR2INT(tcp->dstport)&(0xFFFF)))
		  &&(CHAR2LONG(tcp_sockets[tcp_connections[i].localSocket].ipAddr) == CHAR2LONG(*dstAddr))
		) {
		  	#if DEBUG_TCP
		  		printf("Local Socket %i matches to request\n",i);
		  	#endif
			// check if not-LISTENING, port matching connection exists
			// --> if so, the tcp request belongs to an active connection
			if ((tcp_connections[i].status != TCP_STAT_LISTEN)
			  &&((unsigned int)(tcp_sockets[tcp_connections[i].remoteSocket].port&(0xFFFF)) == ((unsigned int)CHAR2INT(tcp->srcport)&(0xFFFF)))
			  &&(CHAR2LONG(tcp_sockets[tcp_connections[i].remoteSocket].ipAddr) == CHAR2LONG(*srcAddr))
			)
		  	{
				c=i;	
			}
			// check if connection listening. Pick selection only if no
			// port matching, not-listening connection was found yet
			else if ((tcp_connections[i].status == TCP_STAT_LISTEN) 
			         && (c==TCP_MAX_CONNECTIONS))
				c=i;
		}
	}

  	#if DEBUG_TCP
  		if (c!=TCP_MAX_CONNECTIONS)
	  		printf("Connection %i matches request\n",c);
	  	else
	  		print("No Connection found for request\n");

  		printf("Status : %i..",(tcp_connections[c].status));
  		printf("Flags : 0x%02x.. ack?%i\n",tcp->flags,tcp->flags & TCP_ACK_FLAG);
  	#endif

	
	// #########################################################################
	// No connection unvailable... oh well.
	if (c==TCP_MAX_CONNECTIONS){
		// No Ressources available to process
		// --> maybe retransmit will be luckier
	  	#if DEBUG_TCP
	  		print("No Resources for this Request\n");
	  	#endif
		

	// ####### P R O C E S S   R E Q U E S T ###################################
	// After the connection has been determined, check the packet content
	
	// ####### S Y N C H R O N I Z I N G   C O N N E C T I O N S ###############
	// ***************************************************************
	// IF connection (LISTEN) && SYN request?
	} else if ((tcp_connections[c].status==TCP_STAT_LISTEN)
	        && (tcp->flags & TCP_SYN_FLAG)){
	  	#if DEBUG_TCP
	  		printf("Got SYN Packet on Listening Connection %i\n",c);
	  	#endif
	  	// check for Socket space
	  	c_temp = 0;
	  	char socket_free=0;
	  	for (s=0;(s<TCP_MAX_SOCKETS) && (!socket_free);s++){
	  		// for all connections c_temp check if Socket s is in use.
	  		// s is in use if:
	  		//      c_temp.localSocket == s
	  		//   OR c_temp.remoteSocket == s AND c_temp.status != LISTEN
	  		// --> if loop aborts before TCP_MAX_CONNECTIONS is reached
	  		//     the current Socket s is used by c_temp
	  		// TODO: also consider timed out connections to be terminated
	  		//       and free their used socket space	
	  		socket_free=s;
	  		for (c_temp=0;
			     ((c_temp<TCP_MAX_CONNECTIONS)&&socket_free);c_temp++) {
				// check if socket s is used in connection c_temp
				if ((tcp_connections[c_temp].localSocket==s)
			      || ((tcp_connections[c_temp].remoteSocket==s)
			        && (tcp_connections[c_temp].status!=TCP_STAT_LISTEN
			        //TODO: || (tcp_connections[c_temp] TIMED OUT)
			        )))
			    	socket_free=0;
			        
				// TODO: if (tcp_connections[c_temp] TIMED OUT {
				// TODO: 	RESET_CONNECTION(c_temp)
				// TODO: }
			}
		}
	  	// available Socket has been found if s<TCP_MAX_SOCKETS
	  	if (socket_free){
	  		s = socket_free;
		  	#if DEBUG_TCP
		  		printf("available Socket: %i\n",s);
		  	#endif
	  		// set remote socket
	  		tcp_sockets[s].port = CHAR2INT(tcp->srcport);
	  		tcp_sockets[s].ipAddr[0] = (*srcAddr)[0];
	  		tcp_sockets[s].ipAddr[1] = (*srcAddr)[1];
	  		tcp_sockets[s].ipAddr[2] = (*srcAddr)[2];
	  		tcp_sockets[s].ipAddr[3] = (*srcAddr)[3];
	  		tcp_connections[c].remoteSocket = s;
			// generate new Sequence Number (SEQ) and fill TCB
//			tcp_connections[c].iss = TCP_GENERATE_SEQ();
			// let it look random :-)
			tcp_connections[c].iss = (tcp->seq[0])+tcp->seq[1]+(tcp->seq[3]<<8);
		  	tcp_connections[c].irs = 
		  		  ((((unsigned long)(tcp->seq[0]))<<24)&0xFF000000)
		  		+ ((((unsigned long)(tcp->seq[1]))<<16)&0xFF0000)
		  		+ ((((unsigned long)(tcp->seq[2]))<<8)&0xFF00)
		  		+ (tcp->seq[3]&0xFF);
		  	tcp_connections[c].rcv_nxt = tcp_connections[c].irs+1;
			tcp_connections[c].rcv_wnd = TCP_PAYLOAD_SIZE;
			tcp_connections[c].rcv_up = 0;
			// TODO: tcp_connections[c].rcvbuf = ??
			tcp_connections[c].snd_nxt = tcp_connections[c].iss;
			tcp_connections[c].snd_una = 0;
			tcp_connections[c].snd_up = 0;
			// wl1 and wl2 will help to prevent using old segments to update
			// the window (snd_wnd)
			tcp_connections[c].snd_wl1 = CHAR2LONG(tcp->seq);
			tcp_connections[c].snd_wl2 = CHAR2LONG(tcp->ack);
			tcp_connections[c].snd_wnd = ((tcp->window[0]<<8)&0xFF00) 
			                             + (tcp->window[0]&0xFF);
			// set status
			tcp_connections[c].status = TCP_STAT_SYN_RECEIVED;
			
			// TODO: check options
			// TODO: tcp_connections[c].snd_maxsegsize = ?

			// Options have to get their own temporary array because the
			// original option array (within the buffer) might get lost during
			// TCP_Send
			unsigned char options[4];

			options[0] = 0x02;	// Option 02: Maximum Segment Size
			options[1] = 0x04; // Option Length: 4 Byte
			options[2] = ((TCP_PAYLOAD_SIZE)>>8) & 0xFF;
			options[3] = (TCP_PAYLOAD_SIZE) & 0xFF;

		  	#if DEBUG_TCP
		  		print("Sending SYN/ACK\n");
		  	#endif
			// Expected ACK is ONE higher, than initial SEQ-Nr
//			tcp_connections[c].snd_una = tcp_connections[c].snd_nxt+1;
			
			// Sending SYN/ACK
			if (TCP_Send_Ex(
				c,     // Connection
				NULL,  // No Data while Synchronizing
				0,     //  -"-
				(TCP_SYN_FLAG|TCP_ACK_FLAG), //SYN/ACK
				(unsigned char*)options, 
				4,
				0,
				1)>=0){
					
				tcp_connections[c].status = TCP_STAT_ESTABLISHED;
// done in tcp_send now				tcp_connections[c].snd_nxt++;					
			} else {
				tcp_connections[c].status = TCP_STAT_LISTEN;
			};

	  	} else {
		// no socket available for connection request (SYN). Request will
		// be dropped. Maybe retransmission will have more luck later on.
	  	// --> nothing to be done here.
		  	#if DEBUG_TCP
		  		printf("NO available Socket, abort!\n");
		  	#endif
	  	}
    }   	
	// ***************************************************************
	// IF connection (SYN_RECEIVED)&& ACK request
	else if ((tcp_connections[c].status==TCP_STAT_SYN_RECEIVED)
	        && (tcp->flags & TCP_ACK_FLAG)){
	        	
	    if (TCP_Update_Connection(c, tcp, packetlen)==0){

	    	// ACK to own SYN received
	    	
			// Connection is now officially established :-)
/*			tcp_connections[c].status = TCP_STAT_ESTABLISHED;
			tcp_connections[c].snd_nxt++;
		  	#if DEBUG_TCP
		  		print("Connection Established\n");
		  	#endif
*/

		} else {
			// whatever we got there, it wasn't our expected ACK
		}
    }		
	// ***************************************************************
	// I'm not so sure about this case
	// TODO: ELSE IF connection (ESTABLISHED) && SYN request?
		// TODO:?--> send RESET
		// TODO:?--> drop all connection data

	// ***************************************************************
	// TODO: ELSE IF connection (ANY STATE) && receiving RST
		// TODO:--> delete all connection data and set connection to LISTEN
	else if (tcp->flags & TCP_RST_FLAG_OFFSET){
		// reset connection
		tcp_connections[c].status = TCP_STAT_LISTEN;
	}
	// ####### C L O S I N G   C O N N E C T I O N #############################					
	// ***************************************************************
	// ELSE IF connection (ESTABLISHED) && receiving FIN
	else if ((tcp_connections[c].status==TCP_STAT_ESTABLISHED)
	        && (tcp->flags & TCP_FIN_FLAG)){
		
		if (TCP_Update_Connection(c, tcp, packetlen)==0){
		  	#if DEBUG_TCP
		  		print("FIN received\n");
		  	#endif
		  		printf("Closing connection %i to: %02x%02x%02x%02x\n",
		  			c,
		  			tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[0],
		  			tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[1],
		  			tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[2],
		  			tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[3]
		  		);
			// change Status to CLOSE_WAIT
			tcp_connections[c].status = TCP_STAT_CLOSE_WAIT;
//			tcp_connections[c].snd_nxt++;
			tcp_connections[c].rcv_nxt++;
		
			// send ACK
			TCP_Send(c,NULL,0,0,0,0);
			
			// TODO: notify user/app, so he can call TCP_CLOSE
			// lets just assume TCP_CLOSE is instantly being called
			TCP_Close(c);
		}
    }
	        
	// ***************************************************************
	// ELSE IF connection (LAST-ACK) && receiving ACK on CLOSE message
	//      (this status is reached after user called TCP_CLOSE)
	else if ((tcp_connections[c].status==TCP_STAT_LAST_ACK)
	        && (tcp->flags & TCP_ACK_FLAG)
	        && (CHAR2LONG(tcp->ack) == tcp_connections[c].snd_nxt)){

		if (TCP_Update_Connection(c, tcp, packetlen)==0) {	        	
		  	#if DEBUG_TCP
		  		print("Connection closed\n");
		  	#endif
			// TODO: Close Connection, this is (almost) the same as putting it
			// back to LISTEN. This is what we're doing, because we (currently)
			// have a static setup of bound Applications
			tcp_connections[c].status = TCP_STAT_LISTEN;
			
			
		} else {
		}
    }

	// ***************************************************************
	// ELSE IF connection (FIN-WAIT1) && receiving FIN
	//      (this status is reached after user called TCP_CLOSE)
	else if ((tcp_connections[c].status==TCP_STAT_FIN_WAIT1)
			&& (tcp->flags & TCP_ACK_FLAG)
			&& !(tcp->flags & TCP_FIN_FLAG)
			&& (CHAR2LONG(tcp->ack) == tcp_connections[c].snd_nxt)){
		// TODO: possibly more than one ACK until FIN-WAIT2 possible
		// ---> have to wait until conn.snd_nxt == conn.snd_una

		if (TCP_Update_Connection(c, tcp, packetlen)==0) {	        	
			
		  	#if DEBUG_TCP
		  		print("FIN acknowledged\n");
		  	#endif
			tcp_connections[c].status = TCP_STAT_FIN_WAIT2;
		} else {
		}
    }

	// ***************************************************************
	// ELSE IF connection (FIN-WAIT2) && receiving FIN
	//      (this status is reached after user called TCP_CLOSE)
	else if (((tcp_connections[c].status==TCP_STAT_FIN_WAIT1)
			  ||(tcp_connections[c].status==TCP_STAT_FIN_WAIT2))
			&& (tcp->flags & (TCP_FIN_FLAG | TCP_ACK_FLAG))
			&& (CHAR2LONG(tcp->ack) == tcp_connections[c].snd_nxt)){

		if (TCP_Update_Connection(c, tcp, packetlen)==0) {	        	
			
		  	#if DEBUG_TCP
		  		print("FIN acknowledged and received\n");
		  	#endif
			tcp_connections[c].rcv_nxt = CHAR2LONG(tcp->seq)+1;
			tcp_connections[c].status = TCP_STAT_TIMEWAIT;
/*

			// TODO: Close Connection, this is (almost) the same as putting it
			// back to LISTEN. This is what we're doing, because we (currently)
			// have a static setup of binding Applications --> no need to free
			// the local port
			TCP_Send_Ex(c, NULL, 0, TCP_ACK_FLAG, NULL,0,60,0);
			// TODO: while possibly retransmitting we have to wait some
			// time before closing connection 
			tcp_connections[c].status = TCP_STAT_LISTEN;
			
*/
		} else {
		}
    }

	// ####### P R O C E S S   D A T A #########################################
	// ***************************************************************
	// ELSE IF connection (ESTABLISHED) && receiving Data
	//   or connection is still in a close-state that allows receiving data
	else if ((tcp_connections[c].status==TCP_STAT_ESTABLISHED)
	        || (tcp_connections[c].status==TCP_STAT_FIN_WAIT1)
	        || (tcp_connections[c].status==TCP_STAT_FIN_WAIT2))
	{

		if ((TCP_Update_Connection(c, tcp, packetlen)==0)
		   && !TCP_Receive_Data_Blocked
		   // only receive application data if there is any
		   && (TCP_DATA(tcp)>0)
		  /* && (tcp->flags & TCP_PSH_FLAG)*/) {
		  	// Only one application is allowed to receive data at a time.
		  	// This is due to the fact, that we limited RAM causing us to have
		  	//  * only ONE TCP retransmission buffer, which can only be used by
		  	//    one application at a time anyways
		  	//  * problems with overloaded stack which would be caused by
		  	//    multiple TCP_Send routines being stuck because the retrans-
		  	//    mission buffer is full.
		  	//   --> only one application is allowed to send
		   	TCP_Receive_Data_Blocked = 1;

			// launch Application to fetch data
		  	#if DEBUG_TCP
		  		printf("Receiving %i Bytes Application Data: ",TCP_DATA(tcp));
		  	#endif
		  	
		  	// backup snd_nxt state
		  	long tmp_send_nxt = tcp_connections[c].snd_nxt;


			//** HTTP ******
		  	if (tcp_sockets[tcp_connections[c].localSocket].port==TCP_PORT_HTTP){
			  	#if DEBUG_TCP
			  		print("HTTP\n");
			  	#endif
				
				// mark data as received (will be acknowledged with next
				// sent packet on this connection)
				tcp_connections[c].rcv_nxt = CHAR2LONG(tcp->seq)+TCP_DATA(tcp);

			  	testapp_receive(
			  		c,
			  		&(((unsigned char*)tcp)[(tcp->dataoffset>>4)*4]),
			  		packetlen-(tcp->dataoffset>>4)*4
			  	);
			  	
		  	}
		  	//** HTTP:end **
		  	
		  	
		  	// Check if any Data was send, if not send ACK
		  	if (tcp_connections[c].snd_nxt==tmp_send_nxt){
		  		TCP_Send_Ex(c, NULL, 0, TCP_ACK_FLAG, NULL, 0, 60, 0);
		  	}

		   	TCP_Receive_Data_Blocked = 0;
		}

	}

	// ####### O T H E R   R E Q U E S T S #####################################
	// ***************************************************************
	// TODO: ELSE IF no connection exists to socket --> discard
  	
		
	// ***************************************************************
	else {
		// I have no clue how you got here, but it's an unknown connection state
		// --> maybe still something to be done!
		// --> Whatever, lets be nice and return a RESET message. We can't use
		//     TCP_Send for this though, because there is no connection, so the
		//     packet has to be generated manually
	  	#if DEBUG_TCP
	  		printf("Unknown Connection State, send RESET\n");
	  		printf("ack: %04x%04x  snd_nxt: %04x%04x\n",
	  			(int)(CHAR2LONG(tcp->ack)>>16),
	  			(int)(CHAR2LONG(tcp->ack)&0x0000FFFF),
	  			(int)(tcp_connections[c].snd_nxt>>16),
	  			(int)(tcp_connections[c].snd_nxt&0x0000FFFF));
	  	#endif
	  	tcp->flags=TCP_RST_FLAG;
	  	
	  	// swap Ports
	  	TCP_PORT tmpPort = CHAR2INT(tcp->srcport);
	  	tcp->srcport[0] = tcp->dstport[0];
	  	tcp->srcport[1] = tcp->dstport[1];
	  	INT2CHAR2(tmpPort,tcp->dstport);
	  	
	  	// set SEQ and ACK
	  	tcp->seq[0] = tcp->ack[0];tcp->seq[1] = tcp->ack[1];
	  	tcp->seq[2] = tcp->ack[2];tcp->seq[3] = tcp->ack[3];
	  	tcp->ack[0] = 255;tcp->ack[1] = 255;tcp->ack[2] = 255;tcp->ack[3] = 255;
	  	
	  	// save IP Addresses. This is necessarey because the given pointers
	  	// are only temporary and might get lost after calling IP_Send
	  	IP_ADDRESS tmpSrcIp;
	  	IP_ADDRESS tmpDstIp;
	  	tmpDstIp[0] = (*dstAddr)[0];tmpDstIp[1] = (*dstAddr)[1];
	  	tmpDstIp[2] = (*dstAddr)[2];tmpDstIp[3] = (*dstAddr)[3];
	  	tmpSrcIp[0] = (*srcAddr)[0];tmpSrcIp[1] = (*srcAddr)[1];
	  	tmpSrcIp[2] = (*srcAddr)[2];tmpSrcIp[3] = (*srcAddr)[3];

		// calculate Checksum
		tcp->checksum[0]=0;
		tcp->checksum[1]=0;
		sum = 0;
		
		int len = ((sizeof(struct TCP_HEADER)-1) / 4);
		tcp->dataoffset = len<<4;
		len *= 4;

		// Pseudoheader
		// Dest and Src IP Addresses
		sum+=(tmpDstIp[0]<<8)&0xFF00;
		sum+=tmpDstIp[1]&0xFF;
		sum+=(tmpDstIp[2]<<8)&0xFF00;
		sum+=tmpDstIp[3]&0xFF;
		sum+=(tmpSrcIp[0]<<8)&0xFF00;
		sum+=tmpSrcIp[1]&0xFF;
		sum+=(tmpSrcIp[2]<<8)&0xFF00;
		sum+=tmpSrcIp[3]&0xFF;
		// Protocol
		sum+=TCP_PROTOCOL;
		// TCP Length (Header + Data)
		sum+=(len) & 0xFF00;
		sum+=(len) & 0x00FF;
		for (i=0;i<len-1;i=i+2){
			sum +=((((unsigned long)(((unsigned char*)tcp)[i])<<8)&0x0000FF00)+
			              (((unsigned char*)tcp)[i+1]&0x000000FF));
		}
		// if packet length is odd a 0-byte has to be assumed at the end
		if ((len)%2==1){
			sum +=((unsigned long)((((unsigned char*)tcp)[len-1])<<8)&0x0000FF00);
		}
		
		while (sum>>16)
			sum = (sum & 0xFFFF)+(sum >> 16);
	
		sum = ~sum;
		INT2CHAR2(sum,tcp->checksum);
		
		IP_Send(
			(unsigned char*) &tmpSrcIp,  // swap IP Addresses
			(unsigned char*) &tmpDstIp,
			(unsigned char*)(tcp),
			len,
			TCP_PROTOCOL,
			0,
			60);
	  	
	}
  	  	
  	#if DEBUG_CALL
  		print("</TCP_RECEIVE>");
  	#endif
	return 0;  	
}

//############################################################################
signed char TCP_Update_Connection(TCP_CONN_ID c, struct TCP_HEADER* tcp, int packetlen){
//############################################################################

	// checking if incoming SEQ/ACK and Window Data is valid
	if (    ((TCP_DATA(tcp)==0) && (tcp_connections[c].rcv_wnd==0) 
	      && (tcp_connections[c].rcv_nxt==CHAR2LONG(tcp->seq)))
	   || 
	        ((TCP_DATA(tcp)==0) && (tcp_connections[c].rcv_wnd>0)
	      && (tcp_connections[c].rcv_nxt<=CHAR2LONG(tcp->seq)) 
	      && (CHAR2LONG(tcp->seq) < (tcp_connections[c].rcv_nxt+tcp_connections[c].rcv_wnd)))
	   ||
	        ((TCP_DATA(tcp)>0) && (tcp_connections[c].rcv_wnd>0)
	      && ((tcp_connections[c].rcv_nxt <= CHAR2LONG(tcp->seq))
	      && (CHAR2LONG(tcp->seq) < (tcp_connections[c].rcv_nxt+tcp_connections[c].rcv_wnd))))
	      ||((tcp_connections[c].rcv_nxt <= CHAR2LONG(tcp->seq)+TCP_DATA(tcp)-1)
	      && (CHAR2LONG(tcp->seq)+TCP_DATA(tcp)-1 < (tcp_connections[c].rcv_nxt+tcp_connections[c].rcv_wnd)))
	  ){
	     
		// Update Connection
		tcp_connections[c].snd_una = CHAR2LONG(tcp->ack);
		// Update Send Window (see RFC 793, page 71)
		if ((CHAR2INT(tcp->window)!=tcp_connections[c].snd_wnd)
		 &&((tcp_connections[c].snd_wl1 < CHAR2LONG(tcp->seq))
		    ||((tcp_connections[c].snd_wl1 == CHAR2LONG(tcp->seq))
		       &&(tcp_connections[c].snd_wl2 <= CHAR2LONG(tcp->ack))
		      )
		   )
		){
			tcp_connections[c].snd_wnd = CHAR2INT(tcp->window);
		}
				
		return 0;
		
	} else {
	  	#if DEBUG_TCP
	  		print("Received Packet not up to date with SEQ/ACK\n");
	  		printf("SEQ: %04x%04x vs %04x%04x\n",
	  			(unsigned int)(CHAR2LONG(tcp->ack)>>16),
	  			(unsigned int)(0xFFFF&CHAR2LONG(tcp->ack)),
	  			(unsigned int)(tcp_connections[c].snd_nxt>>16),
	  			(unsigned int)(0xFFFF&tcp_connections[c].snd_nxt));
	  		printf("ACK: %04x%04x vs %04x%04x\n",
	  			(unsigned int)(CHAR2LONG(tcp->seq)>>16),
	  			(unsigned int)(0xFFFF&CHAR2LONG(tcp->seq)),
	  			(unsigned int)(tcp_connections[c].rcv_nxt>>16),
	  			(unsigned int)(0xFFFF&tcp_connections[c].rcv_nxt));
	  	#endif
		return -1;
	}

}



// This serves for the user/app to close a TCP connection
//############################################################################
signed TCP_Close(TCP_CONN_ID c){
//############################################################################
	
	// Different resulting states for different connection states
	
	// In case he communications partner had sent a FIN
	// message first, we are in CLOSE_WAIT status now.
	if (tcp_connections[c].status == TCP_STAT_CLOSE_WAIT){
		tcp_connections[c].status = TCP_STAT_LAST_ACK;
	}
		
	// A direct CLOSE call is also possible, of course
	else if (tcp_connections[c].status == TCP_STAT_ESTABLISHED){
		tcp_connections[c].status = TCP_STAT_FIN_WAIT1;
	}
	
	// Closing while still synchronizing or already closing is not possible
	// You call always call RESET to brutely kill a connection
	else return -1;
		
	if (TCP_Send_Ex(
		c,
		NULL,
		0,
		TCP_FIN_FLAG | TCP_ACK_FLAG,
		NULL,
		0,
		0,
		1)>=0){
		
		// After sending FIN snd_nxt has to be increased, even though no data
		// was sent
//		tcp_connections[c].snd_nxt++;
//		tcp_connections[c].rcv_nxt++;

		if ((tcp_connections[c].status==TCP_STAT_LAST_ACK)
	 	  ||(tcp_connections[c].status==TCP_STAT_TIMEWAIT)){
			// TODO: Close Connection, this is (almost) the same as putting it
			// back to LISTEN. This is what we're doing, because we (currently)
			// have a static setup of binding Applications --> no need to free
			// the local port
			TCP_Send_Ex(c, NULL, 0, TCP_ACK_FLAG, NULL,0,60,0);
			// TODO: while possibly retransmitting we have to wait some
			// time before closing connection 
			tcp_connections[c].status = TCP_STAT_LISTEN;
		}
		// else: no FIN received yet --> do nothing and wait for it
			
		return 0;

	} else {
		
		return -1;
	}
		
}


// Send Data through a TCP Connection
//############################################################################
signed char TCP_Send(TCP_CONN_ID c, unsigned char* buffer, int bufferlen, 
//############################################################################
	char psh, char urg, char timeout){

	return TCP_Send_Ex(
		c, 
		buffer, 
		bufferlen, 
		((psh&TCP_PSH_FLAG) | (urg&TCP_URG_FLAG) | TCP_ACK_FLAG),
		NULL,
		0,
		timeout,
		1         // All Standard TCP Send Messages will be retransmitted
	);
}


// Send Data through a TCP Connection knowing more about details of TCP
//############################################################################
signed char TCP_Send_Ex(TCP_CONN_ID c, unsigned char* buffer, int bufferlen, 
//############################################################################
	char Flags, unsigned char* options, int optionlen, char timeout,
	char retransmit){
		
		int i;
		unsigned long sum;
	  	#if DEBUG_CALL
	  		print("<TCP_SEND>");
	  	#endif
	
		// copy buffer to TCP retransmission Buffer
		for (i=0;i<bufferlen;i++){
			// copy Data into Retransmission buffer
			//      --> make sure to add it behind possibly still unacknowledged
			//          Data, which is declared within the c.snd_una field.
			//      --> retrans_buffer_pos = snd_nxt-snd_una
			// TODO:ALSO: make sure the data fits in the buffer, if not return -1
			//      --> maybe another possibility is to add as much data as fits
			//          in the buffer and return a value of how much data was
			//          actually added. It's up to the application to retrans-
			//          mit the leftover then.
			TCP_Retransmission_Buffer[i]=buffer[i];
		}		


	  	
		struct TCP_HEADER * tcp = 
		         (struct TCP_HEADER *) &(reservedMemory[MEM_TCP_SEND_OFFSET]);  	


	  	// Retransmission Loop. Sending is repeated until either the transmission
	  	// was completely acknowledged or the connection was canceled.
	  	unsigned char retry=0;
	  	do {
	  	
			// Build SYN/ACK Reply
			tcp->seq[0] = (tcp_connections[c].snd_nxt>>24)&0xFF;
			tcp->seq[1] = (tcp_connections[c].snd_nxt>>16)&0xFF;
			tcp->seq[2] = (tcp_connections[c].snd_nxt>>8) &0xFF;
			tcp->seq[3] = (tcp_connections[c].snd_nxt)    &0xFF;
			tcp->ack[0] = (tcp_connections[c].rcv_nxt>>24)&0xFF;
			tcp->ack[1] = (tcp_connections[c].rcv_nxt>>16)&0xFF;
			tcp->ack[2] = (tcp_connections[c].rcv_nxt>>8) &0xFF;
			tcp->ack[3] = (tcp_connections[c].rcv_nxt)    &0xFF;
			tcp->dstport[0] = (tcp_sockets[tcp_connections[c].remoteSocket].port>>8)&0xFF;
			tcp->dstport[1] = tcp_sockets[tcp_connections[c].remoteSocket].port&0xFF;
			tcp->srcport[0] = (tcp_sockets[tcp_connections[c].localSocket].port>>8)&0xFF;
			tcp->srcport[1] = tcp_sockets[tcp_connections[c].localSocket].port&0xFF;
		
			// after sending, the Next-Pointer will move up
	/*		if (bufferlen==0)  tcp_connections[c].snd_nxt+=1;
			else*/
			
			tcp->flags = Flags;
			// TODO: support URGENT FLag
			tcp->urgent[0] = 0;
			tcp->urgent[1] = 0;
			INT2CHAR2(tcp_connections[c].rcv_wnd,tcp->window);
	/*				tcp->window[0] = tcp_connections[c].rcv_wnd >>8;
					tcp->window[1] = tcp_connections[c].rcv_wnd >>;*/
	
			// Data offset in 32bit words
			// struct TCP_HEADER includes 4 Bytes for options already so the
			// calculated length of "sizeof(struct TCP_HEADER)+optionlen" would
			// actually be 4 Bytes too long. Minus 1 is still 3 too long.
			// BUT: The Header length is always multiples of 4 so adding 3 and
			// dividing by 4 later, actually makes sure the calculated length is
			// not smaller than it should be. (Remember: Integer division!)
			int len = ((sizeof(struct TCP_HEADER)+optionlen-1) / 4);
			tcp->dataoffset = len<<4;
			len *= 4;
	
			// TCP Options
			for (i=0;i<optionlen;i++){
				tcp->options[i] = options[i];
			}
	
			// Padding: In case the previous calculation increased the header length
			// (up to 3 Bytes possible), these have to be filled with Zeros
			for (i=(sizeof(struct TCP_HEADER)+optionlen-4);i<(len);i++){
				tcp->options[i] = 0;	
			}
			
			// Add Data after Header
			for (i=0;i<bufferlen;i++){
				((unsigned char *)tcp)[len+i]=TCP_Retransmission_Buffer[i];
				// TODO: copy Data into Retransmission buffer
			}
			
			// calculate Checksum
			tcp->checksum[0]=0;
			tcp->checksum[1]=0;
			sum = 0;
			
			// Pseudoheader
			// Dest and Src IP Addresses
			sum+=(tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[0]<<8)&0xFF00;
			sum+=tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[1]&0xFF;
			sum+=(tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[2]<<8)&0xFF00;
			sum+=tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[3]&0xFF;
			sum+=(tcp_sockets[tcp_connections[c].localSocket].ipAddr[0]<<8)&0xFF00;
			sum+=tcp_sockets[tcp_connections[c].localSocket].ipAddr[1]&0xFF;
			sum+=(tcp_sockets[tcp_connections[c].localSocket].ipAddr[2]<<8)&0xFF00;
			sum+=tcp_sockets[tcp_connections[c].localSocket].ipAddr[3]&0xFF;
			// Protocol
			sum+=TCP_PROTOCOL;
			// TCP Length (Header + Data)
			sum+=(len+bufferlen) & 0xFF00;
			sum+=(len+bufferlen) & 0x00FF;
			for (i=0;i<len-1+bufferlen;i=i+2){
				sum +=((((unsigned long)(((unsigned char*)tcp)[i])<<8)&0x0000FF00)+
				              (((unsigned char*)tcp)[i+1]&0x000000FF));
			}
			// if packet length is odd a 0-byte has to be assumed at the end
			if ((len+bufferlen)%2==1){
				sum +=((unsigned long)((((unsigned char*)tcp)[bufferlen+len-1])<<8)&0x0000FF00);
			}
			
			while (sum>>16)
				sum = (sum & 0xFFFF)+(sum >> 16);
		
			sum = ~sum;
			
			INT2CHAR2(sum,tcp->checksum);
	//				tcp->checksum[0]=((int) sum)>>8;
	//				tcp->checksum[0]=(int)(sum&0x000000FF);
							
		  	#if DEBUG_TCP
		  		printf("Ready to send %i Bytes over IP\n",len+bufferlen);
		  	#endif
		  	#if DEBUG_TCP && DEBUG_VERBOSE
		  		debug_print("TCP Data: ",(unsigned char*)tcp,len);
		  	#endif
		  	
		  	#if DEBUG_CALLS
		  		printf("Calling IP_Send with pointers: %i(%02x%02x%02x%02x), %i(%02x%02x%02x%02x), %i\n",
		  			(unsigned int) &(tcp_sockets[tcp_connections[c].remoteSocket].ipAddr),
		  			(tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[0]),
		  			(tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[1]),
		  			(tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[2]),
		  			(tcp_sockets[tcp_connections[c].remoteSocket].ipAddr[3]),
					(unsigned int) &(tcp_sockets[tcp_connections[c].localSocket].ipAddr),
					(tcp_sockets[tcp_connections[c].localSocket].ipAddr[0]),
					(tcp_sockets[tcp_connections[c].localSocket].ipAddr[1]),
					(tcp_sockets[tcp_connections[c].localSocket].ipAddr[2]),
					(tcp_sockets[tcp_connections[c].localSocket].ipAddr[3]),
					(unsigned int)(tcp));
			#endif
		
		
			// set SEQ-Nr of next segment. This number will have to match incoming
			// ACKs
			tcp_connections[c].snd_nxt += bufferlen;	
			if (Flags & (TCP_SYN_FLAG|TCP_FIN_FLAG)) tcp_connections[c].snd_nxt++;

			// Send TCP Packet over IP
			IP_Send(
				(unsigned char*) &(tcp_sockets[tcp_connections[c].remoteSocket].ipAddr),
				(unsigned char*) &(tcp_sockets[tcp_connections[c].localSocket].ipAddr),
				(unsigned char*)(tcp),
				len+bufferlen,
				TCP_PROTOCOL,
				0,
				60);
			
			retry++;
			// Wait some time before retrying. This time is increasing with
			// each retry. Loop is aborted as soon as ACK gets in (Interrupt).
			// TODO: make it binary backoff
			// global Interrupts are switched on for this waiting time
			// IP has to be unblocked in order to receive the ACKs
			IP_BlockReceive=0;
			sei();
			WriteRTL (ISR, (1<<PRX|1<<PTX|1<<RXE|1<<TXE|1<<OVW|1<<CNT|1<<RDC|1<<RST));
			for (i=0;i<retry*TCP_TIMEOUT&&TCP_RETRANSMIT_COND(c,bufferlen);i++);
			cli();
			IP_BlockReceive=1;

			if   ((tcp_connections[c].snd_una < (tcp_connections[c].snd_nxt))
			   ||(tcp_connections[c].status == TCP_STAT_LISTEN)){
			   	
			  	#if DEBUG_TCP
			  		printf("...send(%i) failed\n",retry-1);
			  	#endif
			} else {
			  	#if DEBUG_TCP
			  		printf("...send(%i) ok\n",retry-1);
			  	#endif
			}

	  		// reset snd_nxt, because bufferlen will be readded in
	  		// the next loop (or after the loop)
 			tcp_connections[c].snd_nxt -= bufferlen;	
			if (Flags & (TCP_SYN_FLAG|TCP_FIN_FLAG)) tcp_connections[c].snd_nxt--;

	  	} while (TCP_RETRANSMIT_COND(c,bufferlen) && (retry<TCP_MAX_RETRANSMIT) && retransmit);
	  	
		tcp_connections[c].snd_nxt += bufferlen;
		if (Flags & (TCP_SYN_FLAG|TCP_FIN_FLAG)) tcp_connections[c].snd_nxt++;
	  	
	  	#if DEBUG_TCP
	  		printf("...retries: %i\n",retry-1);
	  		printf("...snd_nxt: %04x%04x snd_una: %04x%04x\n",
	  			(int)(tcp_connections[c].snd_nxt>>16),
	  			(int)(tcp_connections[c].snd_nxt&0x0000FFFF),
	  			(int)(tcp_connections[c].snd_una>>16),
	  			(int)(tcp_connections[c].snd_una&0x0000FFFF));
	  		printf("return\n");
	  	#endif
		
	  	#if DEBUG_CALL
	  		print("</TCP_SEND>");
	  	#endif
		return (tcp_connections[c].snd_una>=tcp_connections[c].snd_nxt) ? 0 :-1;
}

