/*#######################################################################################

AVR Embedded Webservice Project

File        : tcp.h

Description : Implementation of the TCP protocol (header).

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#ifndef _TCP_H_
#define _TCP_H_

#include "main.h"

#define TCP_MAX_HEADER_SIZE   60
#define TCP_PAYLOAD_SIZE      (MTU-IP_HEADER_SIZE-TCP_MAX_HEADER_SIZE)
#define TCP_SIZE              TCP_PAYLOAD_SIZE
#define TCP_SEND_SIZE         10
#define TCP_PROTOCOL          0x06
#define TCP_MAX_CONNECTIONS   1
#define TCP_MAX_SOCKETS       2

// maximum count of one packet's retransmissions
#define TCP_MAX_RETRANSMIT    5
// check if packet was acknowledged ...
// ... and connection is still alive
#define TCP_RETRANSMIT_COND(c,b) ((tcp_connections[c].snd_una < (tcp_connections[c].snd_nxt+b)) \
                               &&(tcp_connections[c].status != TCP_STAT_LISTEN))
// Timeout for TCP retransmissions (for-loops)
#define TCP_TIMEOUT           10000

#define TCP_STAT_LISTEN       1
#define TCP_STAT_SYN_SENT     2
#define TCP_STAT_SYN_RECEIVED 3
#define TCP_STAT_ESTABLISHED  4
#define TCP_STAT_FIN_WAIT1    5
#define TCP_STAT_FIN_WAIT2    6
#define TCP_STAT_CLOSE_WAIT   7
#define TCP_STAT_CLOSING      8
#define TCP_STAT_LAST_ACK     9
#define TCP_STAT_TIMEWAIT    10

#define TCP_FIN_FLAG_OFFSET   0
#define TCP_SYN_FLAG_OFFSET   1
#define TCP_RST_FLAG_OFFSET   2
#define TCP_PSH_FLAG_OFFSET   3
#define TCP_ACK_FLAG_OFFSET   4
#define TCP_URG_FLAG_OFFSET   5

#define TCP_FIN_FLAG          1  //(1<<TCP_FIN_FLAG_OFFSET)
#define TCP_SYN_FLAG          2  //(1<<TCP_SYN_FLAG_OFFSET)
#define TCP_RST_FLAG          4  //(1<<TCP_RST_FLAG_OFFSET)
#define TCP_PSH_FLAG          8  //(1<<TCP_PSH_FLAG_OFFSET)
#define TCP_ACK_FLAG          16 //(1<<TCP_ACK_FLAG_OFFSET)
#define TCP_URG_FLAG          32 //(1<<TCP_URG_FLAG_OFFSET)

#define TCP_GENERATE_SEQ()    time;
#define TCP_DATA(tcpbuffer)   (packetlen - (tcpbuffer->dataoffset>>4)*4)

typedef unsigned long SEQ;
typedef unsigned char TCP_CONN_ID;
typedef unsigned char TCP_SCKT_ID;
typedef unsigned int  TCP_PORT;
typedef unsigned char TCP_STATUS;


// TCP_HEADER
struct TCP_HEADER{
	unsigned char srcport[2];
	unsigned char dstport[2];
	unsigned char seq[4];
	unsigned char ack[4];
	unsigned char dataoffset; // upper 4 bits is count the # of 32Bit words in Header
	unsigned char flags;
	unsigned char window[2];
	unsigned char checksum[2];
	unsigned char urgent[2];
	unsigned char options[4]; // upper bytes are Options, lower bytes padding
};

struct TCP_SOCKET{
	IP_ADDRESS ipAddr;
	TCP_PORT port;
};

// Transmission Control Block, any TCP connection has to have one
struct TCB {
	TCP_SCKT_ID localSocket;      // index of local Socket Array
	TCP_SCKT_ID remoteSocket;     // index of local Socket Array
	TCP_STATUS status;     
	unsigned char *sendbuf;
	unsigned char *rcvbuf;
	unsigned char *retransmitqueue;

	// Send Sequence Variables
	SEQ snd_una;
	SEQ snd_nxt;
	unsigned int snd_wnd;
	unsigned int snd_maxsegsize;
	SEQ snd_up;
	SEQ snd_wl1;
	SEQ snd_wl2;
	SEQ iss;
	// Receive Sequence Variables		
	SEQ rcv_nxt;
	unsigned int rcv_wnd;
	SEQ rcv_up;
	SEQ irs;
	
};


// Only one Application at a time is allowed to send and receive data
// to prevent stack from overrunning and having multiple applications waiting
// to send data over one TCP Retransmission Buffer.
// --> TCP_Receive_Data_Blocked == 1 whenever an app is processing data
extern char TCP_Receive_Data_Blocked;

extern struct TCB tcp_connections[];
//struct TCP_SOCKET *tcp_sockets;
extern struct TCP_SOCKET tcp_sockets[];
extern unsigned char TCP_Retransmission_Buffer[];

extern signed char TCP_Receive(IP_ADDRESS *dstAddr, IP_ADDRESS *srcAddr, 
  unsigned char* packet,int packetlen);
  
extern signed TCP_Close(TCP_CONN_ID c);

extern signed char TCP_Send(TCP_CONN_ID c, unsigned char* buffer, 
  	int bufferlen, char PUSH, char URGENT, char timeout);

signed char TCP_Send_Ex(TCP_CONN_ID c, unsigned char* buffer, int bufferlen, 
	char Flags, unsigned char* options, int optionlen, char timeout,
	char retransmit);

signed char TCP_Update_Connection(TCP_CONN_ID c, struct TCP_HEADER* tcp, 
	int packetlen);

#endif //_TCP_H_
