/*#######################################################################################

AVR Embedded Webservice Project

File        : udp.c

Description : Implementation of the UDP protocol.

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Chris Helbing, helbing@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de
Stefan Wanja, wanja@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#######################################################################################*/


#include "main.h"

//############################################################################
signed char UDP_Receive( IP_ADDRESS* destAddr,IP_ADDRESS* sourceAddr,
                               unsigned char* packet,int packetlen){
//############################################################################
	#if DEBUG_CALLS
		printf("---- UDP_RECEIVE\n");
	#endif
	struct UDP_HEADER *udp;
	udp = (struct UDP_HEADER *) packet;
	// check if we have a listening socket on this packet's destPort
	// --> later...
	// at the moment we only need UDP for DHCP, so its hardcoded
	if(CHAR2INT(udp->destPort)==UDP_PORT_DHCPCLIENT){		
		#if DEBUG_UDP
			printf("DHCP_RECEIVE packet found.\n");
		#endif
		DHCP_Receive(
				&((unsigned char *)udp)[UDP_HEADER_SIZE],
				packetlen-UDP_HEADER_SIZE
			);
	}
	return 0;
}

//############################################################################
signed char UDP_Send(struct UDP_SOCKET* dstSocket, struct UDP_SOCKET* srcSocket,
	                 unsigned char* packet, int packetlen){
//############################################################################
	unsigned long sum;
	unsigned int i;
	#if DEBUG_CALLS
		printf("---- UDP_SEND\n");
	#endif
	struct UDP_HEADER* udp = 
		(struct UDP_HEADER*)&(reservedMemory[MEM_UDP_SEND_OFFSET]);
	for(int i=0;i<packetlen;((unsigned char *)udp)[UDP_HEADER_SIZE+i]=packet[i++]);
	udp->sourcePort[0]=srcSocket->port[0];
	udp->sourcePort[1]=srcSocket->port[1];
	udp->destPort[0]=dstSocket->port[0];
	udp->destPort[1]=dstSocket->port[1];
	// Set UDP-Length
	udp->length[0]=(packetlen+UDP_HEADER_SIZE)>>8;
	udp->length[1]=(packetlen+UDP_HEADER_SIZE) & 0x00FF;
	// Checksum 
		udp->checksum[0]=0x00;
		udp->checksum[1]=0x00;
		// calc checksum
		sum = 0;
		for (i=0;i<UDP_HEADER_SIZE+packetlen-1;i=i+2){
			sum +=(((unsigned long)((((unsigned char *)udp)[i])<<8)&0x0000FF00)+
	              (packet[i+1]&0x000000FF));
		}
		// PSEUDO HEADER
		sum+=(srcSocket->ip[0]<<8)+srcSocket->ip[1];
		sum+=(srcSocket->ip[2]<<8)+srcSocket->ip[3];
		sum+=(dstSocket->ip[0]<<8)+dstSocket->ip[1];
		sum+=(dstSocket->ip[2]<<8)+dstSocket->ip[3];		
		sum+=(UDP_PROTOCOL);	
		sum+=packetlen+UDP_HEADER_SIZE+12; //+12 for pseudo header length
		
		// if packet length is odd a 0-byte has to be assumed at the end
		if ((packetlen+UDP_HEADER_SIZE)%2==1){
			sum +=((unsigned long)((((unsigned char *)udp)[packetlen+UDP_HEADER_SIZE-1])<<8));//&0x0000FF00);
		}
		// take only 16 bits out of the 32 bit sum and add up the carries
		sum = (sum & 0x0000FFFF)+(sum >> 16);
		sum = (sum & 0x0000FFFF)+(sum >> 16);
		sum = ~sum;
		sum=0;
	// set checksum
	udp->checksum[0]=sum>>8;
	udp->checksum[1]=sum & 0x00FF;
	#ifdef DEBUG_UDP
		printf("src: %i.%i.%i.%i\ndst: %i.%i.%i.%i\n",srcSocket->ip[0],srcSocket->ip[1],
		srcSocket->ip[1],srcSocket->ip[2],dstSocket->ip[0],dstSocket->ip[1],
		dstSocket->ip[1],dstSocket->ip[2]);
	#endif
	// Send the stuff
	IP_Send((unsigned char*)&dstSocket->ip,(unsigned char*)&srcSocket->ip,(unsigned char*)udp,packetlen+UDP_HEADER_SIZE,UDP_PROTOCOL,0,10);
	return 0;
}
